@@ -46,7 +46,8 @@
 
 // auto generated ui files
 #include "ui_modonhdwidget.h"
-#include "ui_appearanceconfigwidget.h"
+#include "ui_textareaappearanceconfigwidget.h"
+#include "ui_bordersappearanceconfigwidget.h"
 #include "ui_cursorconfigwidget.h"
 #include "ui_editconfigwidget.h"
 #include "ui_indentationconfigwidget.h"
@@ -169,11 +170,16 @@ KateIndentConfigTab::KateIndentConfigTab(QWidget *parent)
   //
 
   connect(ui->cmbMode, SIGNAL(activated(int)), this, SLOT(slotChanged()));
+  connect(ui->rbIndentWithTabs, SIGNAL(toggled(bool)), this, SLOT(slotChanged()));
+  connect(ui->rbIndentWithSpaces, SIGNAL(toggled(bool)), this, SLOT(slotChanged()));
+  connect(ui->rbIndentMixed, SIGNAL(toggled(bool)), this, SLOT(slotChanged()));
+  connect(ui->rbIndentWithTabs, SIGNAL(toggled(bool)), ui->sbIndentWidth, SLOT(setDisabled(bool)));
 
   connect(ui->chkKeepExtraSpaces, SIGNAL(toggled(bool)), this, SLOT(slotChanged()));
   connect(ui->chkIndentPaste, SIGNAL(toggled(bool)), this, SLOT(slotChanged()));
   connect(ui->chkBackspaceUnindents, SIGNAL(toggled(bool)), this, SLOT(slotChanged()));
 
+  connect(ui->sbTabWidth, SIGNAL(valueChanged(int)), this, SLOT(slotChanged()));
   connect(ui->sbIndentWidth, SIGNAL(valueChanged(int)), this, SLOT(slotChanged()));
 
   connect(ui->rbTabAdvances, SIGNAL(toggled(bool)), this, SLOT(slotChanged()));
@@ -189,6 +195,14 @@ KateIndentConfigTab::~KateIndentConfigTab()
   delete ui;
 }
 
+void KateIndentConfigTab::slotChanged()
+{
+  if (ui->rbIndentWithTabs->isChecked())
+    ui->sbIndentWidth->setValue(ui->sbTabWidth->value());
+
+  KateConfigPage::slotChanged();
+}
+
 void KateIndentConfigTab::showWhatsThis(const QString& text)
 {
   QWhatsThis::showText(QCursor::pos(), text);
@@ -203,19 +217,13 @@ void KateIndentConfigTab::apply ()
 
   KateDocumentConfig::global()->configStart ();
 
-  uint configFlags = KateDocumentConfig::global()->configFlags();
-
-  configFlags &= ~KateDocumentConfig::cfKeepExtraSpaces;
-  configFlags &= ~KateDocumentConfig::cfIndentPastedText;
-  configFlags &= ~KateDocumentConfig::cfBackspaceIndents;
-
-  if (ui->chkKeepExtraSpaces->isChecked()) configFlags |= KateDocumentConfig::cfKeepExtraSpaces;
-  if (ui->chkIndentPaste->isChecked()) configFlags |= KateDocumentConfig::cfIndentPastedText;
-  if (ui->chkBackspaceUnindents->isChecked()) configFlags |= KateDocumentConfig::cfBackspaceIndents;
-
-  KateDocumentConfig::global()->setConfigFlags(configFlags);
+  KateDocumentConfig::global()->setKeepExtraSpaces(ui->chkKeepExtraSpaces->isChecked());
+  KateDocumentConfig::global()->setBackspaceIndents(ui->chkBackspaceUnindents->isChecked());
+  KateDocumentConfig::global()->setIndentPastedText(ui->chkIndentPaste->isChecked());
   KateDocumentConfig::global()->setIndentationWidth(ui->sbIndentWidth->value());
   KateDocumentConfig::global()->setIndentationMode(KateAutoIndent::modeName(ui->cmbMode->currentIndex()));
+  KateDocumentConfig::global()->setTabWidth(ui->sbTabWidth->value());
+  KateDocumentConfig::global()->setReplaceTabsDyn(ui->rbIndentWithSpaces->isChecked());
 
   if (ui->rbTabAdvances->isChecked())
     KateDocumentConfig::global()->setTabHandling( KateDocumentConfig::tabInsertsTab );
@@ -229,19 +237,31 @@ void KateIndentConfigTab::apply ()
 
 void KateIndentConfigTab::reload ()
 {
-  uint configFlags = KateDocumentConfig::global()->configFlags();
-
+  ui->sbTabWidth->setSuffix(ki18np(" character", " characters"));
+  ui->sbTabWidth->setValue(KateDocumentConfig::global()->tabWidth());
   ui->sbIndentWidth->setSuffix(ki18np(" character", " characters"));
   ui->sbIndentWidth->setValue(KateDocumentConfig::global()->indentationWidth());
-  ui->chkKeepExtraSpaces->setChecked(configFlags & KateDocumentConfig::cfKeepExtraSpaces);
-  ui->chkIndentPaste->setChecked(configFlags & KateDocumentConfig::cfIndentPastedText);
-  ui->chkBackspaceUnindents->setChecked(configFlags & KateDocumentConfig::cfBackspaceIndents);
+  ui->chkKeepExtraSpaces->setChecked(KateDocumentConfig::global()->keepExtraSpaces());
+  ui->chkIndentPaste->setChecked(KateDocumentConfig::global()->indentPastedText());
+  ui->chkBackspaceUnindents->setChecked(KateDocumentConfig::global()->backspaceIndents());
 
   ui->rbTabAdvances->setChecked( KateDocumentConfig::global()->tabHandling() == KateDocumentConfig::tabInsertsTab );
   ui->rbTabIndents->setChecked( KateDocumentConfig::global()->tabHandling() == KateDocumentConfig::tabIndents );
   ui->rbTabSmart->setChecked( KateDocumentConfig::global()->tabHandling() == KateDocumentConfig::tabSmart );
 
   ui->cmbMode->setCurrentIndex (KateAutoIndent::modeNumber (KateDocumentConfig::global()->indentationMode()));
+
+  if (KateDocumentConfig::global()->replaceTabsDyn())
+    ui->rbIndentWithSpaces->setChecked (true);
+  else
+  {
+    if (KateDocumentConfig::global()->indentationWidth() == KateDocumentConfig::global()->tabWidth())
+      ui->rbIndentWithTabs->setChecked (true);
+    else
+      ui->rbIndentMixed->setChecked (true);
+  }
+
+  ui->sbIndentWidth->setEnabled(!ui->rbIndentWithTabs->isChecked());
 }
 //END KateIndentConfigTab
 
@@ -456,8 +476,8 @@ void KateSpellCheckConfigTab::showWhatsThis(const QString& text)
 
 void KateSpellCheckConfigTab::apply()
 {
-// nothing changed, no need to apply stuff
   if (!hasChanged()) {
+    // nothing changed, no need to apply stuff
     return;
   }
   m_changed = false;
@@ -465,10 +485,8 @@ void KateSpellCheckConfigTab::apply()
   KateDocumentConfig::global()->configStart();
   m_sonnetConfigWidget->save();
   KateDocumentConfig::global()->configEnd();
-  const QList<KateDocument*>& kateDocumentList = KateGlobal::self()->kateDocuments();
-  for (QList<KateDocument*>::const_iterator it = kateDocumentList.begin();
-       it != kateDocumentList.end(); ++it) {
-    (*it)->refreshOnTheFlyCheck();
+  foreach (KateDocument *doc, KateGlobal::self()->kateDocuments()) {
+    doc->refreshOnTheFlyCheck();
   }
 }
 
@@ -524,15 +542,8 @@ void KateSelectConfigTab::apply ()
   KateViewConfig::global()->configStart ();
   KateDocumentConfig::global()->configStart ();
 
-  uint configFlags = KateDocumentConfig::global()->configFlags();
-
-  configFlags &= ~KateDocumentConfig::cfSmartHome;
-  configFlags &= ~KateDocumentConfig::cfWrapCursor;
-
-  if (ui->chkSmartHome->isChecked()) configFlags |= KateDocumentConfig::cfSmartHome;
-  if (ui->chkWrapCursor->isChecked()) configFlags |= KateDocumentConfig::cfWrapCursor;
-
-  KateDocumentConfig::global()->setConfigFlags(configFlags);
+  KateDocumentConfig::global()->setSmartHome(ui->chkSmartHome->isChecked());
+  KateDocumentConfig::global()->setWrapCursor(ui->chkWrapCursor->isChecked());
 
   KateViewConfig::global()->setAutoCenterLines(qMax(0, ui->sbAutoCenterCursor->value()));
   KateDocumentConfig::global()->setPageUpDownMovesCursor(ui->chkPagingMovesCursor->isChecked());
@@ -548,10 +559,8 @@ void KateSelectConfigTab::reload ()
   ui->rbNormal->setChecked( ! KateViewConfig::global()->persistentSelection() );
   ui->rbPersistent->setChecked( KateViewConfig::global()->persistentSelection() );
 
-  const uint configFlags = KateDocumentConfig::global()->configFlags();
-
-  ui->chkSmartHome->setChecked(configFlags & KateDocumentConfig::cfSmartHome);
-  ui->chkWrapCursor->setChecked(configFlags & KateDocumentConfig::cfWrapCursor);
+  ui->chkSmartHome->setChecked(KateDocumentConfig::global()->smartHome());
+  ui->chkWrapCursor->setChecked(KateDocumentConfig::global()->wrapCursor());
   ui->chkPagingMovesCursor->setChecked(KateDocumentConfig::global()->pageUpDownMovesCursor());
   ui->sbAutoCenterCursor->setValue(KateViewConfig::global()->autoCenterLines());
 }
@@ -568,10 +577,6 @@ KateEditGeneralConfigTab::KateEditGeneralConfigTab(QWidget *parent)
 
   reload();
 
-  connect( ui->chkReplaceTabs, SIGNAL(toggled(bool)), this, SLOT(slotChanged()) );
-  connect(ui->chkShowTabs, SIGNAL(toggled(bool)), this, SLOT(slotChanged()));
-  connect(ui->chkShowSpaces, SIGNAL(toggled(bool)), this, SLOT(slotChanged()));
-  connect(ui->sbTabWidth, SIGNAL(valueChanged(int)), this, SLOT(slotChanged()));
   connect(ui->chkStaticWordWrap, SIGNAL(toggled(bool)), this, SLOT(slotChanged()));
   connect(ui->chkShowStaticWordWrapMarker, SIGNAL(toggled(bool)), this, SLOT(slotChanged()));
   connect(ui->sbWordWrap, SIGNAL(valueChanged(int)), this, SLOT(slotChanged()));
@@ -601,25 +606,11 @@ void KateEditGeneralConfigTab::apply ()
   KateViewConfig::global()->configStart ();
   KateDocumentConfig::global()->configStart ();
 
-  uint configFlags = KateDocumentConfig::global()->configFlags();
-
-  configFlags &= ~KateDocumentConfig::cfAutoBrackets;
-  configFlags &= ~KateDocumentConfig::cfShowTabs;
-  configFlags &= ~KateDocumentConfig::cfShowSpaces;
-  configFlags &= ~KateDocumentConfig::cfReplaceTabsDyn;
-  configFlags &= ~KateDocumentConfig::cfRemoveTrailingDyn;
-
-  if (ui->chkAutoBrackets->isChecked()) configFlags |= KateDocumentConfig::cfAutoBrackets;
-  if (ui->chkShowTabs->isChecked()) configFlags |= KateDocumentConfig::cfShowTabs;
-  if (ui->chkShowSpaces->isChecked()) configFlags |= KateDocumentConfig::cfShowSpaces;
-  if (ui->chkReplaceTabs->isChecked()) configFlags |= KateDocumentConfig::cfReplaceTabsDyn;
-  if (ui->chkRemoveTrailingSpaces->isChecked()) configFlags |= KateDocumentConfig::cfRemoveTrailingDyn;
-
-  KateDocumentConfig::global()->setConfigFlags(configFlags);
+  KateDocumentConfig::global()->setAutoBrackets(ui->chkAutoBrackets->isChecked());
+  KateDocumentConfig::global()->setRemoveTrailingDyn(ui->chkRemoveTrailingSpaces->isChecked());
 
   KateDocumentConfig::global()->setWordWrapAt(ui->sbWordWrap->value());
   KateDocumentConfig::global()->setWordWrap(ui->chkStaticWordWrap->isChecked());
-  KateDocumentConfig::global()->setTabWidth(ui->sbTabWidth->value());
 
   KateRendererConfig::global()->setWordWrapMarker (ui->chkShowStaticWordWrapMarker->isChecked());
 
@@ -631,19 +622,12 @@ void KateEditGeneralConfigTab::apply ()
 
 void KateEditGeneralConfigTab::reload ()
 {
-  uint configFlags = KateDocumentConfig::global()->configFlags();
-
-  ui->chkReplaceTabs->setChecked( configFlags & KateDocumentConfig::cfReplaceTabsDyn );
-  ui->chkShowTabs->setChecked( configFlags & KateDocumentConfig::cfShowTabs );
-  ui->chkShowSpaces->setChecked( configFlags & KateDocumentConfig::cfShowSpaces );
-  ui->sbTabWidth->setSuffix(ki18np(" character", " characters"));
-  ui->sbTabWidth->setValue( KateDocumentConfig::global()->tabWidth() );
   ui->chkStaticWordWrap->setChecked(KateDocumentConfig::global()->wordWrap());
   ui->chkShowStaticWordWrapMarker->setChecked( KateRendererConfig::global()->wordWrapMarker() );
-  ui->sbWordWrap->setSuffix(ki18np(" character", " characters"));
+  ui->sbWordWrap->setSuffix(ki18ncp("Wrap words at", " character", " characters"));
   ui->sbWordWrap->setValue( KateDocumentConfig::global()->wordWrapAt() );
-  ui->chkRemoveTrailingSpaces->setChecked( configFlags & KateDocumentConfig::cfRemoveTrailingDyn );
-  ui->chkAutoBrackets->setChecked( configFlags & KateDocumentConfig::cfAutoBrackets );
+  ui->chkRemoveTrailingSpaces->setChecked( KateDocumentConfig::global()->removeTrailingDyn() );
+  ui->chkAutoBrackets->setChecked( KateDocumentConfig::global()->autoBrackets() );
   ui->chkSmartCopyCut->setChecked( KateViewConfig::global()->smartCopyCut() );
   ui->chkScrollPastEnd->setChecked( KateViewConfig::global()->scrollPastEnd() );
 }
@@ -731,17 +715,33 @@ void KateEditConfigTab::defaults ()
 
 //BEGIN KateViewDefaultsConfig
 KateViewDefaultsConfig::KateViewDefaultsConfig(QWidget *parent)
-  :KateConfigPage(parent)
+  : KateConfigPage(parent)
+  , textareaUi(new Ui::TextareaAppearanceConfigWidget())
+  , bordersUi(new Ui::BordersAppearanceConfigWidget())
 {
-  ui = new Ui::AppearanceConfigWidget();
-  ui->setupUi( this );
+  QLayout *layout = new QVBoxLayout( this );
+  QTabWidget *tabWidget = new QTabWidget( this );
+  layout->addWidget( tabWidget );
+  layout->setMargin( 0 );
+
+  QWidget *textareaTab = new QWidget( tabWidget );
+  textareaUi->setupUi( textareaTab );
+  tabWidget->addTab( textareaTab, i18n("General") );
+
+  QWidget *bordersTab = new QWidget( tabWidget );
+  bordersUi->setupUi( bordersTab );
+  tabWidget->addTab( bordersTab, i18n("Borders") );
 
   if (KateDocument::simpleMode ())
-    ui->gbSortBookmarks->hide ();
+    bordersUi->gbSortBookmarks->hide ();
 
-  ui->cmbDynamicWordWrapIndicator->addItem( i18n("Off") );
-  ui->cmbDynamicWordWrapIndicator->addItem( i18n("Follow Line Numbers") );
-  ui->cmbDynamicWordWrapIndicator->addItem( i18n("Always On") );
+  textareaUi->cmbDynamicWordWrapIndicator->addItem( i18n("Off") );
+  textareaUi->cmbDynamicWordWrapIndicator->addItem( i18n("Follow Line Numbers") );
+  textareaUi->cmbDynamicWordWrapIndicator->addItem( i18n("Always On") );
+
+  // hide power user mode if activated anyway
+  if (!KateGlobal::self()->simpleMode ())
+    textareaUi->chkDeveloperMode->hide ();
 
   // What's This? help is in the ui-file
 
@@ -751,23 +751,27 @@ KateViewDefaultsConfig::KateViewDefaultsConfig(QWidget *parent)
   // after initial reload, connect the stuff for the changed () signal
   //
 
-  connect(ui->gbWordWrap, SIGNAL(toggled(bool)), this, SLOT(slotChanged()));
-  connect(ui->cmbDynamicWordWrapIndicator, SIGNAL(activated(int)), this, SLOT(slotChanged()));
-  connect(ui->sbDynamicWordWrapDepth, SIGNAL(valueChanged(int)), this, SLOT(slotChanged()));
-  connect(ui->chkIconBorder, SIGNAL(toggled(bool)), this, SLOT(slotChanged()));
-  connect(ui->chkScrollbarMarks, SIGNAL(toggled(bool)), this, SLOT(slotChanged()));
-  connect(ui->chkLineNumbers, SIGNAL(toggled(bool)), this, SLOT(slotChanged()));
-  connect(ui->chkShowFoldingMarkers, SIGNAL(toggled(bool)), this, SLOT(slotChanged()));
-  connect(ui->rbSortBookmarksByPosition, SIGNAL(toggled(bool)), this, SLOT(slotChanged()));
-  connect(ui->rbSortBookmarksByCreation, SIGNAL(toggled(bool)), this, SLOT(slotChanged()));
-  connect(ui->chkShowIndentationLines, SIGNAL(toggled(bool)), this, SLOT(slotChanged()));
-  connect(ui->chkShowWholeBracketExpression, SIGNAL(toggled(bool)), this, SLOT(slotChanged()));
-  connect(ui->chkDeveloperMode, SIGNAL(toggled(bool)), this, SLOT(slotChanged()));
+  connect(textareaUi->gbWordWrap, SIGNAL(toggled(bool)), this, SLOT(slotChanged()));
+  connect(textareaUi->cmbDynamicWordWrapIndicator, SIGNAL(activated(int)), this, SLOT(slotChanged()));
+  connect(textareaUi->sbDynamicWordWrapDepth, SIGNAL(valueChanged(int)), this, SLOT(slotChanged()));
+  connect(textareaUi->chkShowTabs, SIGNAL(toggled(bool)), this, SLOT(slotChanged()));
+  connect(textareaUi->chkShowSpaces, SIGNAL(toggled(bool)), this, SLOT(slotChanged()));
+  connect(textareaUi->chkShowIndentationLines, SIGNAL(toggled(bool)), this, SLOT(slotChanged()));
+  connect(textareaUi->chkShowWholeBracketExpression, SIGNAL(toggled(bool)), this, SLOT(slotChanged()));
+  connect(textareaUi->chkDeveloperMode, SIGNAL(toggled(bool)), this, SLOT(slotChanged()));
+
+  connect(bordersUi->chkIconBorder, SIGNAL(toggled(bool)), this, SLOT(slotChanged()));
+  connect(bordersUi->chkScrollbarMarks, SIGNAL(toggled(bool)), this, SLOT(slotChanged()));
+  connect(bordersUi->chkLineNumbers, SIGNAL(toggled(bool)), this, SLOT(slotChanged()));
+  connect(bordersUi->chkShowFoldingMarkers, SIGNAL(toggled(bool)), this, SLOT(slotChanged()));
+  connect(bordersUi->rbSortBookmarksByPosition, SIGNAL(toggled(bool)), this, SLOT(slotChanged()));
+  connect(bordersUi->rbSortBookmarksByCreation, SIGNAL(toggled(bool)), this, SLOT(slotChanged()));
 }
 
 KateViewDefaultsConfig::~KateViewDefaultsConfig()
 {
-  delete ui;
+  delete bordersUi;
+  delete textareaUi;
 }
 
 void KateViewDefaultsConfig::apply ()
@@ -780,20 +784,22 @@ void KateViewDefaultsConfig::apply ()
   KateViewConfig::global()->configStart ();
   KateRendererConfig::global()->configStart ();
 
-  KateViewConfig::global()->setDynWordWrap (ui->gbWordWrap->isChecked());
-  KateViewConfig::global()->setDynWordWrapIndicators (ui->cmbDynamicWordWrapIndicator->currentIndex ());
-  KateViewConfig::global()->setDynWordWrapAlignIndent(ui->sbDynamicWordWrapDepth->value());
-  KateViewConfig::global()->setLineNumbers (ui->chkLineNumbers->isChecked());
-  KateViewConfig::global()->setIconBar (ui->chkIconBorder->isChecked());
-  KateViewConfig::global()->setScrollBarMarks (ui->chkScrollbarMarks->isChecked());
-  KateViewConfig::global()->setFoldingBar (ui->chkShowFoldingMarkers->isChecked());
+  KateViewConfig::global()->setDynWordWrap (textareaUi->gbWordWrap->isChecked());
+  KateViewConfig::global()->setDynWordWrapIndicators (textareaUi->cmbDynamicWordWrapIndicator->currentIndex ());
+  KateViewConfig::global()->setDynWordWrapAlignIndent(textareaUi->sbDynamicWordWrapDepth->value());
+  KateDocumentConfig::global()->setShowTabs (textareaUi->chkShowTabs->isChecked());
+  KateDocumentConfig::global()->setShowSpaces (textareaUi->chkShowSpaces->isChecked());
+  KateViewConfig::global()->setLineNumbers (bordersUi->chkLineNumbers->isChecked());
+  KateViewConfig::global()->setIconBar (bordersUi->chkIconBorder->isChecked());
+  KateViewConfig::global()->setScrollBarMarks (bordersUi->chkScrollbarMarks->isChecked());
+  KateViewConfig::global()->setFoldingBar (bordersUi->chkShowFoldingMarkers->isChecked());
 
-  KateViewConfig::global()->setBookmarkSort (ui->rbSortBookmarksByPosition->isChecked()?0:1);
-  KateRendererConfig::global()->setShowIndentationLines(ui->chkShowIndentationLines->isChecked());
-  KateRendererConfig::global()->setShowWholeBracketExpression(ui->chkShowWholeBracketExpression->isChecked());
+  KateViewConfig::global()->setBookmarkSort (bordersUi->rbSortBookmarksByPosition->isChecked()?0:1);
+  KateRendererConfig::global()->setShowIndentationLines(textareaUi->chkShowIndentationLines->isChecked());
+  KateRendererConfig::global()->setShowWholeBracketExpression(textareaUi->chkShowWholeBracketExpression->isChecked());
 
   // warn user that he needs restart the application
-  if (!ui->chkDeveloperMode->isChecked() != KateDocumentConfig::global()->allowSimpleMode())
+  if (!textareaUi->chkDeveloperMode->isChecked() != KateDocumentConfig::global()->allowSimpleMode())
   {
     // inform...
     KMessageBox::information(
@@ -801,7 +807,7 @@ void KateViewDefaultsConfig::apply ()
                 i18n("Changing the power user mode affects only newly opened / created documents. In KWrite a restart is recommended."),
                 i18n("Power user mode changed"));
 
-    KateDocumentConfig::global()->setAllowSimpleMode (!ui->chkDeveloperMode->isChecked());
+    KateDocumentConfig::global()->setAllowSimpleMode (!textareaUi->chkDeveloperMode->isChecked());
   }
 
   KateRendererConfig::global()->configEnd ();
@@ -810,18 +816,20 @@ void KateViewDefaultsConfig::apply ()
 
 void KateViewDefaultsConfig::reload ()
 {
-  ui->gbWordWrap->setChecked(KateViewConfig::global()->dynWordWrap());
-  ui->cmbDynamicWordWrapIndicator->setCurrentIndex( KateViewConfig::global()->dynWordWrapIndicators() );
-  ui->sbDynamicWordWrapDepth->setValue(KateViewConfig::global()->dynWordWrapAlignIndent());
-  ui->chkLineNumbers->setChecked(KateViewConfig::global()->lineNumbers());
-  ui->chkIconBorder->setChecked(KateViewConfig::global()->iconBar());
-  ui->chkScrollbarMarks->setChecked(KateViewConfig::global()->scrollBarMarks());
-  ui->chkShowFoldingMarkers->setChecked(KateViewConfig::global()->foldingBar());
-  ui->rbSortBookmarksByPosition->setChecked(KateViewConfig::global()->bookmarkSort()==0);
-  ui->rbSortBookmarksByCreation->setChecked(KateViewConfig::global()->bookmarkSort()==1);
-  ui->chkShowIndentationLines->setChecked(KateRendererConfig::global()->showIndentationLines());
-  ui->chkShowWholeBracketExpression->setChecked(KateRendererConfig::global()->showWholeBracketExpression());
-  ui->chkDeveloperMode->setChecked(!KateDocumentConfig::global()->allowSimpleMode());
+  textareaUi->gbWordWrap->setChecked(KateViewConfig::global()->dynWordWrap());
+  textareaUi->cmbDynamicWordWrapIndicator->setCurrentIndex( KateViewConfig::global()->dynWordWrapIndicators() );
+  textareaUi->sbDynamicWordWrapDepth->setValue(KateViewConfig::global()->dynWordWrapAlignIndent());
+  textareaUi->chkShowTabs->setChecked(KateDocumentConfig::global()->showTabs());
+  textareaUi->chkShowSpaces->setChecked(KateDocumentConfig::global()->showSpaces());
+  bordersUi->chkLineNumbers->setChecked(KateViewConfig::global()->lineNumbers());
+  bordersUi->chkIconBorder->setChecked(KateViewConfig::global()->iconBar());
+  bordersUi->chkScrollbarMarks->setChecked(KateViewConfig::global()->scrollBarMarks());
+  bordersUi->chkShowFoldingMarkers->setChecked(KateViewConfig::global()->foldingBar());
+  bordersUi->rbSortBookmarksByPosition->setChecked(KateViewConfig::global()->bookmarkSort()==0);
+  bordersUi->rbSortBookmarksByCreation->setChecked(KateViewConfig::global()->bookmarkSort()==1);
+  textareaUi->chkShowIndentationLines->setChecked(KateRendererConfig::global()->showIndentationLines());
+  textareaUi->chkShowWholeBracketExpression->setChecked(KateRendererConfig::global()->showWholeBracketExpression());
+  textareaUi->chkDeveloperMode->setChecked(!KateDocumentConfig::global()->allowSimpleMode());
 }
 
 void KateViewDefaultsConfig::reset () {;}
@@ -862,6 +870,7 @@ KateSaveConfigTab::KateSaveConfigTab( QWidget *parent )
 
   connect( ui->cmbEncoding, SIGNAL(activated(int)), this, SLOT(slotChanged()));
   connect( ui->cmbEncodingDetection, SIGNAL(activated(int)), this, SLOT(slotChanged()));
+  connect( ui->cmbEncodingFallback, SIGNAL(activated(int)), this, SLOT(slotChanged()));
   connect( ui->cmbEOL, SIGNAL(activated(int)), this, SLOT(slotChanged()));
   connect( ui->chkDetectEOL, SIGNAL( toggled(bool) ), this, SLOT( slotChanged() ) );
   connect( ui->chkEnableBOM, SIGNAL( toggled(bool) ), this, SLOT( slotChanged() ) );
@@ -871,6 +880,7 @@ KateSaveConfigTab::KateSaveConfigTab( QWidget *parent )
   connect( uiadv->sbConfigFileSearchDepth, SIGNAL(valueChanged(int)), this, SLOT(slotChanged()));
   connect( uiadv->edtBackupPrefix, SIGNAL( textChanged ( const QString & ) ), this, SLOT( slotChanged() ) );
   connect( uiadv->edtBackupSuffix, SIGNAL( textChanged ( const QString & ) ), this, SLOT( slotChanged() ) );
+  connect( uiadv->chkNoSync, SIGNAL( toggled(bool) ), this, SLOT( slotChanged() ) );
 
   internalLayout->addWidget(newWidget);
   tmpWidget->setLayout(internalLayout);
@@ -902,6 +912,7 @@ void KateSaveConfigTab::apply()
     return;
   m_changed = false;
 
+  KateGlobalConfig::global()->configStart ();
   KateDocumentConfig::global()->configStart ();
 
   if ( uiadv->edtBackupSuffix->text().isEmpty() && uiadv->edtBackupPrefix->text().isEmpty() ) {
@@ -923,34 +934,35 @@ void KateSaveConfigTab::apply()
   KateDocumentConfig::global()->setBackupPrefix(uiadv->edtBackupPrefix->text());
   KateDocumentConfig::global()->setBackupSuffix(uiadv->edtBackupSuffix->text());
 
+  KateDocumentConfig::global()->setSwapFileNoSync(uiadv->chkNoSync->isChecked());
+
   KateDocumentConfig::global()->setSearchDirConfigDepth(uiadv->sbConfigFileSearchDepth->value());
 
-  uint configFlags = KateDocumentConfig::global()->configFlags();
-
-  configFlags &= ~KateDocumentConfig::cfRemoveSpaces; // clear flag
-  if (ui->chkRemoveTrailingSpaces->isChecked()) configFlags |= KateDocumentConfig::cfRemoveSpaces; // set flag if checked
-
-  KateDocumentConfig::global()->setConfigFlags(configFlags);
+  KateDocumentConfig::global()->setRemoveSpaces(ui->chkRemoveTrailingSpaces->isChecked());
 
+  // set both standard and fallback encoding
   KateDocumentConfig::global()->setEncoding((ui->cmbEncoding->currentIndex() == 0) ? "" : KGlobal::charsets()->encodingForName(ui->cmbEncoding->currentText()));
-  KateDocumentConfig::global()->setEncodingProberType(
-      (KEncodingProber::ProberType)ui->cmbEncodingDetection->itemData(ui->cmbEncodingDetection->currentIndex()).toUInt());
+
+  KateGlobalConfig::global()->setProberType((KEncodingProber::ProberType)ui->cmbEncodingDetection->currentIndex());
+  KateGlobalConfig::global()->setFallbackEncoding(KGlobal::charsets()->encodingForName(ui->cmbEncodingFallback->currentText()));
 
   KateDocumentConfig::global()->setEol(ui->cmbEOL->currentIndex());
   KateDocumentConfig::global()->setAllowEolDetection(ui->chkDetectEOL->isChecked());
   KateDocumentConfig::global()->setBom(ui->chkEnableBOM->isChecked());
 
   KateDocumentConfig::global()->configEnd ();
+  KateGlobalConfig::global()->configEnd ();
 }
 
 void KateSaveConfigTab::reload()
 {
   modeConfigPage->reload();
 
-  // encoding
+  // encodings
   ui->cmbEncoding->clear ();
   ui->cmbEncoding->addItem (i18n("KDE Default"));
   ui->cmbEncoding->setCurrentIndex(0);
+  ui->cmbEncodingFallback->clear ();
   QStringList encodings (KGlobal::charsets()->descriptiveEncodingNames());
   int insert = 1;
   for (int i=0; i < encodings.count(); i++)
@@ -961,42 +973,43 @@ void KateSaveConfigTab::reload()
     if (found)
     {
       ui->cmbEncoding->addItem (encodings[i]);
+      ui->cmbEncodingFallback->addItem (encodings[i]);
 
       if ( codecForEnc->name() == KateDocumentConfig::global()->encoding() )
       {
         ui->cmbEncoding->setCurrentIndex(insert);
       }
 
+      if ( codecForEnc == KateGlobalConfig::global()->fallbackCodec() )
+      {
+        // adjust index for fallback config, has no default!
+        ui->cmbEncodingFallback->setCurrentIndex(insert-1);
+      }
+
       insert++;
     }
   }
 
   // encoding detection
   ui->cmbEncodingDetection->clear ();
-
-  ui->cmbEncodingDetection->addItem (i18n("Disabled"), QVariant((uint)KEncodingProber::None));
-  ui->cmbEncodingDetection->setCurrentIndex(0);
-
-  ui->cmbEncodingDetection->addItem (i18n("Universal"), QVariant((uint)KEncodingProber::Universal));
-
-  QStringList items;
-  foreach (const QStringList &encodingsForScript, KGlobal::charsets()->encodingsByScript())
-    items << encodingsForScript.at(0);
-  items.sort();
-  foreach (const QString &item, items) {
-    KEncodingProber::ProberType scri=KEncodingProber::proberTypeForName(item);
-    ui->cmbEncodingDetection->addItem (item, QVariant((uint)scri));
-    if (scri==KateDocumentConfig::global()->encodingProberType())
+  bool found = false;
+  for (int i = 0; !KEncodingProber::nameForProberType ((KEncodingProber::ProberType) i).isEmpty(); ++i) {
+    ui->cmbEncodingDetection->addItem (KEncodingProber::nameForProberType ((KEncodingProber::ProberType) i));
+    if (i == KateGlobalConfig::global()->proberType()) {
       ui->cmbEncodingDetection->setCurrentIndex(ui->cmbEncodingDetection->count()-1);
+      found = true;
+    }
   }
+  if (!found)
+      ui->cmbEncodingDetection->setCurrentIndex(KEncodingProber::Universal);
+
 
   // eol
   ui->cmbEOL->setCurrentIndex(KateDocumentConfig::global()->eol());
   ui->chkDetectEOL->setChecked(KateDocumentConfig::global()->allowEolDetection());
   ui->chkEnableBOM->setChecked(KateDocumentConfig::global()->bom());
-  
-  const uint configFlags = KateDocumentConfig::global()->configFlags();
-  ui->chkRemoveTrailingSpaces->setChecked(configFlags & KateDocumentConfig::cfRemoveSpaces);
+
+  ui->chkRemoveTrailingSpaces->setChecked(KateDocumentConfig::global()->removeSpaces());
   uiadv->sbConfigFileSearchDepth->setValue(KateDocumentConfig::global()->searchDirConfigDepth());
 
   // other stuff
@@ -1005,6 +1018,7 @@ void KateSaveConfigTab::reload()
   uiadv->chkBackupRemoteFiles->setChecked( f & KateDocumentConfig::RemoteFiles );
   uiadv->edtBackupPrefix->setText( KateDocumentConfig::global()->backupPrefix() );
   uiadv->edtBackupSuffix->setText( KateDocumentConfig::global()->backupSuffix() );
+  uiadv->chkNoSync->setChecked( KateDocumentConfig::global()->swapFileNoSync() );
 }
 
 void KateSaveConfigTab::reset()
@@ -1020,6 +1034,7 @@ void KateSaveConfigTab::defaults()
   uiadv->chkBackupRemoteFiles->setChecked( false );
   uiadv->edtBackupPrefix->setText( "" );
   uiadv->edtBackupSuffix->setText( "~" );
+  uiadv->chkNoSync->setChecked( false );
 }
 
 //END KateSaveConfigTab
@@ -1165,7 +1180,6 @@ KateHlDownloadDialog::KateHlDownloadDialog(QWidget *parent, const char *name, bo
   setDefaultButton( User1 );
   setObjectName( name );
   setModal( modal );
-  showButtonSeparator( true );
 
   KVBox* vbox = new KVBox(this);
   setMainWidget(vbox);
@@ -1283,9 +1297,12 @@ void KateHlDownloadDialog::slotUser1()
 //END KateHlDownloadDialog
 
 //BEGIN KateGotoBar
-KateGotoBar::KateGotoBar(KateView* view, QWidget *parent)
-  : KateViewBarWidget( true, view, parent )
+KateGotoBar::KateGotoBar(KTextEditor::View *view, QWidget *parent)
+  : KateViewBarWidget( true, parent )
+  , m_view( view )
 {
+  Q_ASSERT( m_view != 0 );  // this bar widget is pointless w/o a view
+
   QHBoxLayout *topLayout = new QHBoxLayout( centralWidget() );
   topLayout->setMargin(0);
   gotoRange = new KIntSpinBox(centralWidget());
@@ -1293,7 +1310,7 @@ KateGotoBar::KateGotoBar(KateView* view, QWidget *parent)
   QLabel *label = new QLabel(i18n("&Go to line:"), centralWidget() );
   label->setBuddy(gotoRange);
 
-  btnOK = new QToolButton();
+  QToolButton *btnOK = new QToolButton(centralWidget());
   btnOK->setAutoRaise(true);
   btnOK->setIcon(QIcon(SmallIcon("go-jump")));
   btnOK->setText(i18n("Go"));
@@ -1305,17 +1322,16 @@ KateGotoBar::KateGotoBar(KateView* view, QWidget *parent)
   topLayout->setStretchFactor( gotoRange, 0 );
   topLayout->addWidget(btnOK);
   topLayout->addStretch();
+
+  setFocusProxy(gotoRange);
 }
 
 void KateGotoBar::updateData()
 {
-  if (!view())
-    return;
-
-  gotoRange->setMaximum(view()->doc()->lines());
+  gotoRange->setMaximum(m_view->document()->lines());
   if (!isVisible())
   {
-    gotoRange->setValue(view()->cursorPosition().line() + 1);
+    gotoRange->setValue(m_view->cursorPosition().line() + 1);
     gotoRange->adjustSize(); // ### does not respect the range :-(
   }
   gotoRange->setFocus(Qt::OtherFocusReason);
@@ -1334,16 +1350,18 @@ void KateGotoBar::keyPressEvent(QKeyEvent* event)
 
 void KateGotoBar::gotoLine()
 {
-  view()->setCursorPosition( KTextEditor::Cursor(gotoRange->value() - 1, 0) );
-  view()->setFocus();
+  m_view->setCursorPosition( KTextEditor::Cursor(gotoRange->value() - 1, 0) );
+  m_view->setFocus();
   emit hideMe();
 }
 //END KateGotoBar
 
 //BEGIN KateDictionaryBar
 KateDictionaryBar::KateDictionaryBar(KateView* view, QWidget *parent)
-  : KateViewBarWidget( true, view, parent )
+  : KateViewBarWidget( true, parent )
+  , m_view( view )
 {
+  Q_ASSERT(m_view != 0); // this bar widget is pointless w/o a view
 
   QHBoxLayout *topLayout = new QHBoxLayout(centralWidget());
   topLayout->setMargin(0);
@@ -1368,11 +1386,7 @@ KateDictionaryBar::~KateDictionaryBar()
 
 void KateDictionaryBar::updateData()
 {
-  if (!view()) {
-    return;
-  }
-
-  KateDocument *document = view()->doc();
+  KateDocument *document = m_view->doc();
   QString dictionary = document->defaultDictionary();
   if(dictionary.isEmpty()) {
     dictionary = Sonnet::Speller().defaultLanguage();
@@ -1382,17 +1396,12 @@ void KateDictionaryBar::updateData()
 
 void KateDictionaryBar::dictionaryChanged(const QString& dictionary)
 {
-  KateView *kateView = view();
-  if (!kateView) {
-    return;
-  }
-
-  KTextEditor::Range selection = kateView->selectionRange();
+  KTextEditor::Range selection = m_view->selectionRange();
   if(selection.isValid() && !selection.isEmpty()) {
-    kateView->doc()->setDictionary(dictionary, selection);
+    m_view->doc()->setDictionary(dictionary, selection);
   }
   else {
-    kateView->doc()->setDefaultDictionary(dictionary);
+    m_view->doc()->setDefaultDictionary(dictionary);
   }
 }
 
@@ -1433,7 +1442,6 @@ KateModOnHdPrompt::KateModOnHdPrompt( KateDocument *doc,
   setButtonWhatsThis( Cancel, i18n("Do nothing. Next time you focus the file, "
       "or try to save it or close it, you will be prompted again.") );
 
-  showButtonSeparator( true );
   setCaption( title );
 
   QWidget *w = new QWidget(this);
