@@ -178,6 +178,9 @@ void KateScrollBar::sliderChange ( SliderChange change )
 
 void KateScrollBar::wheelEvent(QWheelEvent *e)
 {
+  if (e->modifiers() == Qt::NoModifier)
+    return QScrollBar::wheelEvent(e);
+
   QCoreApplication::sendEvent(m_viewInternal, e);
 }
 
@@ -299,6 +302,11 @@ void KateCommandLineBar::setText(const QString &text, bool selected)
   }
 }
 
+void KateCommandLineBar::execute(const QString &text)
+{
+  m_lineEdit->slotReturnPressed(text);
+}
+
 KateCmdLineEdit::KateCmdLineEdit (KateCommandLineBar *bar, KateView *view)
   : KLineEdit ()
   , m_view (view)
@@ -501,9 +509,14 @@ void KateCmdLineEdit::slotReturnPressed ( const QString& text )
         }
         else
         {
-          if (msg.length() > 0)
-            setText (i18n ("Error: ") + msg);
-          else
+          if (msg.length() > 0) {
+            if (msg.contains('\n')) {
+              // multiline error, use widget with more space
+              QWhatsThis::showText(mapToGlobal(QPoint(0,0)), msg);
+            } else {
+              setText(msg);
+            }
+          } else
             setText (i18n ("Command \"%1\" failed.",  cmd));
           KNotification::beep();
         }
@@ -532,8 +545,9 @@ void KateCmdLineEdit::slotReturnPressed ( const QString& text )
     m_view->setFocus ();
   }
 
-  if (isVisible())
+  if (isVisible()) {
     m_hideTimer->start(4000);
+  }
 }
 
 void KateCmdLineEdit::hideLineEdit () // unless i have focus ;)
@@ -1003,7 +1017,6 @@ static void paintTriangle (QPainter &painter, const QColor &baseColor, int xOffs
 
 void KateIconBorder::paintBorder (int /*x*/, int y, int /*width*/, int height)
 {
-  QMutexLocker lock(m_view->m_doc->smartMutex());
   uint h = m_view->renderer()->config()->fontMetrics().height();
   uint startz = (y / h);
   uint endz = startz + 1 + (height / h);
@@ -1415,12 +1428,13 @@ void KateIconBorder::mouseReleaseEvent( QMouseEvent* e )
     if( area == IconBorder) {
       if (e->button() == Qt::LeftButton) {
         if( !m_doc->handleMarkClick(cursorOnLine) ) {
-          if( m_doc->editableMarks() & KateViewConfig::global()->defaultMarkType() ) {
-            if( m_doc->mark( cursorOnLine ) & KateViewConfig::global()->defaultMarkType() )
-              m_doc->removeMark( cursorOnLine, KateViewConfig::global()->defaultMarkType() );
+          KateViewConfig *config = m_view->config();
+          if( m_doc->editableMarks() & config->defaultMarkType() ) {
+            if( m_doc->mark( cursorOnLine ) & config->defaultMarkType() )
+              m_doc->removeMark( cursorOnLine, config->defaultMarkType() );
             else
-              m_doc->addMark( cursorOnLine, KateViewConfig::global()->defaultMarkType() );
-            } else {
+              m_doc->addMark( cursorOnLine, config->defaultMarkType() );
+            } else if (config->allowMarkMenu()) {
               showMarkMenu( cursorOnLine, QCursor::pos() );
             }
         }
@@ -1472,6 +1486,9 @@ void KateIconBorder::mouseDoubleClickEvent( QMouseEvent* e )
 
 void KateIconBorder::showMarkMenu( uint line, const QPoint& pos )
 {
+  if( !m_view->config()->allowMarkMenu() )
+    return;
+    
   if( m_doc->handleMarkContextMenu( line, pos ) )
     return;
 
@@ -1842,32 +1859,13 @@ KateViewBarWidget::KateViewBarWidget (bool addCloseButton, QWidget *parent)
   setFocusProxy(m_centralWidget);
 }
 
-KateStackedWidget::KateStackedWidget(QWidget* parent)
-  : QStackedWidget(parent)
-{}
-
-QSize KateStackedWidget::sizeHint() const
-{
-  if (currentWidget())
-    return currentWidget()->sizeHint();
-  return QStackedWidget::sizeHint();
-}
-
-QSize KateStackedWidget::minimumSize() const
-{
-  if (currentWidget())
-    return currentWidget()->minimumSize();
-  return QStackedWidget::minimumSize();
-}
-
-
 
 KateViewBar::KateViewBar (bool external,KTextEditor::ViewBarContainer::Position pos,QWidget *parent, KateView *view)
  : QWidget (parent), m_external(external), m_pos(pos),m_view (view), m_permanentBarWidget(0)
 
 {
   m_layout = new QVBoxLayout(this);
-  m_stack = new KateStackedWidget(this);
+  m_stack = new QStackedWidget(this);
   m_layout->addWidget(m_stack);
   m_layout->setMargin(0);
 
@@ -1877,7 +1875,7 @@ KateViewBar::KateViewBar (bool external,KTextEditor::ViewBarContainer::Position
 
 void KateViewBar::addBarWidget (KateViewBarWidget *newBarWidget)
 {
-  if (hasWidget(newBarWidget)) {
+  if (hasBarWidget(newBarWidget)) {
     kDebug(13025) << "this bar widget is already added";
     return;
   }
@@ -1889,6 +1887,11 @@ void KateViewBar::addBarWidget (KateViewBarWidget *newBarWidget)
   kDebug(13025)<<"add barwidget " << newBarWidget;
 }
 
+void KateViewBar::removeBarWidget (KateViewBarWidget *barWidget)
+{
+  m_stack->removeWidget(barWidget);
+}
+
 void KateViewBar::addPermanentBarWidget (KateViewBarWidget *barWidget)
 {
   // remove old widget from layout (if any)
@@ -1930,9 +1933,12 @@ bool KateViewBar::hasPermanentWidget (KateViewBarWidget *barWidget ) const
 
 void KateViewBar::showBarWidget (KateViewBarWidget *barWidget)
 {
+  Q_ASSERT(barWidget != 0);
+
   // raise correct widget
   m_stack->setCurrentWidget (barWidget);
   barWidget->show();
+  barWidget->setFocus(Qt::ShortcutFocusReason);
   m_stack->show();
 
   // if we have any permanent widget, bar is always visible,
@@ -1942,15 +1948,9 @@ void KateViewBar::showBarWidget (KateViewBarWidget *barWidget)
   }
 }
 
-bool KateViewBar::hasWidget(KateViewBarWidget* wid) const
+bool KateViewBar::hasBarWidget(KateViewBarWidget* barWidget) const
 {
-    int count = m_stack->count();
-    for (int i=0; i<count; ++i) {
-        if (m_stack->widget(i) == wid) {
-            return true;
-        }
-    }
-    return false;
+  return m_stack->indexOf(barWidget) != -1;
 }
 
 void KateViewBar::hideCurrentBarWidget ()
