@@ -527,13 +527,13 @@ QChar KCharsets::fromEntity(const QString &str)
         return QChar::Null;
 
     int pos = 0;
-    if(str[pos] == '&') pos++;
+    if(str[pos] == QLatin1Char('&')) pos++;
 
     // Check for '&#000' or '&#x0000' sequence
-    if (str[pos] == '#' && str.length()-pos > 1) {
+    if (str[pos] == QLatin1Char('#') && str.length()-pos > 1) {
         bool ok;
         pos++;
-        if (str[pos] == 'x' || str[pos] == 'X') {
+        if (str[pos] == QLatin1Char('x') || str[pos] == QLatin1Char('X')) {
             pos++;
             // '&#x0000', hexadecimal character reference
             const QString tmp( str.mid( pos ) );
@@ -596,13 +596,13 @@ QString KCharsets::resolveEntities( const QString &input )
     for ( ; p < end; ++p ) {
         const QChar ch = *p;
 
-        if ( ch == '&' ) {
+        if ( ch == QLatin1Char('&') ) {
             ampersand = p;
             scanForSemicolon = true;
             continue;
         }
 
-        if ( ch != ';' || scanForSemicolon == false )
+        if ( ch != QLatin1Char(';') || scanForSemicolon == false )
             continue;
 
         assert( ampersand );
@@ -640,6 +640,7 @@ QStringList KCharsets::availableEncodingNames() const
     return available;
 }
 
+#ifndef KDE_NO_DEPRECATED
 QString KCharsets::languageForEncoding( const QString &encoding ) const
 {
     const char* lang = kcharsets_array_search( (const char*)language_for_encoding_string,
@@ -650,6 +651,7 @@ QString KCharsets::languageForEncoding( const QString &encoding ) const
     else
         return i18nc( "@item Text character set", "Other" );
 }
+#endif
 
 QString KCharsets::descriptionForEncoding( const QString& encoding ) const
 {
@@ -665,14 +667,14 @@ QString KCharsets::descriptionForEncoding( const QString& encoding ) const
 
 QString KCharsets::encodingForName( const QString &descriptiveName ) const
 {
-    const int left = descriptiveName.lastIndexOf( '(' );
+    const int left = descriptiveName.lastIndexOf( QLatin1Char('(') );
 
     if (left<0) // No parenthesis, so assume it is a normal encoding name
 	return descriptiveName.trimmed();
 
     QString name(descriptiveName.mid(left+1));
 
-    const int right = name.lastIndexOf( ')' );
+    const int right = name.lastIndexOf( QLatin1Char(')') );
 
     if (right<0)
         return name;
@@ -719,6 +721,8 @@ QList<QStringList> KCharsets::encodingsByScript() const
 
 QTextCodec* KCharsets::codecForName(const QString &n) const
 {
+    if ( n == QLatin1String("gb2312") || n == QLatin1String("gbk") )
+        return QTextCodec::codecForName( "gb18030" );
     const QByteArray name( n.toLatin1() );
     QTextCodec* codec = codecForNameOrNull( name );
     if ( codec )
@@ -729,6 +733,10 @@ QTextCodec* KCharsets::codecForName(const QString &n) const
 
 QTextCodec* KCharsets::codecForName(const QString &n, bool &ok) const
 {
+    if (n == QLatin1String("gb2312") || n == QLatin1String("gbk")) {
+        ok = true;
+        return QTextCodec::codecForName( "gb18030" );
+    }
     const QByteArray name( n.toLatin1() );
     QTextCodec* codec = codecForNameOrNull( name );
     if ( codec )
@@ -773,11 +781,11 @@ QTextCodec *KCharsets::codecForNameOrNull( const QByteArray& n ) const
 
     QByteArray name = n.toLower();
     bool changed = false;
-    if (name.endsWith("_charset")) {
+    if (name.endsWith("_charset")) { // krazy:exclude=strings
        name.chop( 8 );
        changed = true;
     }
-    if ( name.startsWith( "x-" ) ) {
+    if ( name.startsWith( "x-" ) ) { // krazy:exclude=strings
        name.remove( 0, 2 ); // remove x- at start
        changed = true;
     }
@@ -814,7 +822,7 @@ QTextCodec *KCharsets::codecForNameOrNull( const QByteArray& n ) const
 #warning is it still useful with Qt4 ?
 #endif
 	//don't forget to remove the #if 0 on a few structs at the top also if you reenable that ;)  (search for 852 )
-	//from what I understood, one needs to create a QTextCodecPlugin in order to be able to support a new Codec, but I do not 
+	//from what I understood, one needs to create a QTextCodecPlugin in order to be able to support a new Codec, but I do not
 	//know how to convert a charmap to a QTextCodec and the real big question is whether we need that at all ...  (mikmak)
         // Yes, it is useful (for examples EBCDIC in Kate or codepages for KOffice filters from/to MS formats) (goutte)
 #if 0
