@@ -46,6 +46,7 @@ private Q_SLOTS:
     void testEnums();
     void testEntryMap();
     void testInvalid();
+    void testDeleteEntry();
     void testDelete();
     void testDeleteWhenLocalized();
     void testDefaultGroup();
@@ -71,6 +72,7 @@ private Q_SLOTS:
     void testOptionOrder();
     void testLocaleConfig();
     void testKdeGlobals();
+    void testNoKdeHome();
 
     // unrelated
     void testKAboutDataOrganizationDomain();
@@ -79,7 +81,6 @@ private Q_SLOTS:
     void testSyncOnExit();
 private:
     void revertEntries();
-    static QList<QByteArray> readLines(const char* fileName = "kconfigtest");
 };
 Q_DECLARE_OPERATORS_FOR_FLAGS(KConfigTest::Flags)
 
