@@ -26,7 +26,8 @@ class KLocaleTest : public QObject
 	Q_OBJECT
 private Q_SLOTS:
 	void initTestCase();
-	void formatNumbers();
+    void languages();
+    void formatNumbers();
 	void formatNumberStrings();
 	void readNumber();
 	void formatDate();
@@ -34,10 +35,15 @@ private Q_SLOTS:
 	void formatTime();
 	void formatDateTime();
 	void readTime();
+    void testDayPeriods();
+    void testCalendarSystemType();
+    void testCountryDivision();
 	void prettyFormatDuration();
+	void formatByteSize();
 	void bug95511();
 	void weekDays();
 	void removeAcceleratorMarker();
+	void formatByteSize2();
 };
 
 #endif // KLOCALETEST_H
