@@ -173,10 +173,12 @@ void KShortcutsEditor::clearConfiguration()
 }
 
 
+#ifndef KDE_NO_DEPRECATED
 void KShortcutsEditor::importConfiguration( KConfig *config)
 {
     d->importConfiguration(config);
 }
+#endif
 
 
 void KShortcutsEditor::importConfiguration( KConfigBase *config)
@@ -185,10 +187,12 @@ void KShortcutsEditor::importConfiguration( KConfigBase *config)
 }
 
 
+#ifndef KDE_NO_DEPRECATED
 void KShortcutsEditor::exportConfiguration( KConfig *config) const
 {
     exportConfiguration(static_cast<KConfigBase*>(config));
 }
+#endif
 
 
 void KShortcutsEditor::exportConfiguration( KConfigBase *config) const
@@ -334,7 +338,7 @@ bool KShortcutsEditorPrivate::addAction(QAction *action, QTreeWidgetItem *hier[]
     // If the action name starts with unnamed- spit out a warning and ignore
     // it. That name will change at will and will break loading and writing
     QString actionName = action->objectName();
-    if (actionName.isEmpty() || actionName.startsWith("unnamed-")) {
+    if (actionName.isEmpty() || actionName.startsWith(QLatin1String("unnamed-"))) {
         kError() << "Skipping action without name " << action->text() << "," << actionName << "!";
         return false;
     }
@@ -432,7 +436,7 @@ void KShortcutsEditorPrivate::changeKeyShortcut(KShortcutsEditorItem *item, uint
     }
 
     item->setKeySequence(column, capture);
-    q->keyChange();
+     q->keyChange();
     //force view update
     item->setText(column, capture.toString(QKeySequence::NativeText));
 }
@@ -536,7 +540,7 @@ void KShortcutsEditorPrivate::importConfiguration(KConfigBase *config)
 
             KShortcutsEditorItem *item = static_cast<KShortcutsEditorItem *>(*it);
 
-            QString actionName = item->data(Name).toString();
+            QString actionName = item->data(Id).toString();
             KShortcut sc(globalShortcutsGroup.readEntry(actionName, QString()));
             changeKeyShortcut(item, GlobalPrimary, sc.primary());
         }
@@ -613,6 +617,8 @@ more important things):
 */
 void KShortcutsEditorPrivate::printShortcuts() const
 {
+// One cant print on wince
+#ifndef _WIN32_WCE
     QTreeWidgetItem* root = ui.list->invisibleRootItem();
     QTextDocument doc;
     doc.setDefaultFont(KGlobalSettings::generalFont());
@@ -716,6 +722,7 @@ void KShortcutsEditorPrivate::printShortcuts() const
         doc.print(&printer);
     }
     delete dlg;
+#endif
 }
 
 #include "kshortcutseditor.moc"
