@@ -37,6 +37,7 @@
 #include <QtCore/QBuffer>
 #include <QtCore/QDataStream>
 #include <QtCore/QByteArray>
+#include <QtCore/QStringBuilder> // % operator for QString
 #include <QtGui/QIcon>
 #include <QtGui/QPainter>
 #include <QMovie>
@@ -48,7 +49,9 @@
 #include <kglobal.h>
 #include <kglobalsettings.h>
 #include <kcomponentdata.h>
+#ifndef _WIN32_WCE
 #include <ksvgrenderer.h>
+#endif
 #include <kde_file.h>
 #include <kshareddatacache.h>
 
@@ -59,6 +62,9 @@
 #include <assert.h>
 #include <kconfiggroup.h>
 
+// Used to make cache keys for icons with no group. Result type is QString*
+K_GLOBAL_STATIC_WITH_ARGS(QString, NULL_EFFECT_FINGERPRINT, (QString::fromLatin1("noeffect")))
+
 // Qt implements Tiny SVG specification. This specification does not cover important elements
 // that are pretty globally used on our icons, like blurring (and other filters). TT seems to have
 // no interest in supporting the full SVG specification (it would be slower, even with JS, CSS
@@ -491,7 +497,7 @@ void KIconLoaderPrivate::init( const QString& _appname, KStandardDirs *_dirs )
 
     // loading config and default sizes
     mpGroups = new KIconGroup[(int) KIconLoader::LastGroup];
-    for (KIconLoader::Group i=KIconLoader::FirstGroup; i<KIconLoader::LastGroup; i++)
+    for (KIconLoader::Group i=KIconLoader::FirstGroup; i<KIconLoader::LastGroup; ++i)
     {
         if (groups[i] == 0L)
             break;
@@ -747,8 +753,13 @@ void KIconLoaderPrivate::normalizeIconMetadata(KIconLoader::Group &group, int &s
     }
 
     // If size == 0, use default size for the specified group.
-    if (size == 0 && group != KIconLoader::User)
+    if (size == 0)
     {
+        if (group < 0)
+        {
+            kWarning(264) << "Neither size nor group specified!";
+            group = KIconLoader::Desktop;
+        }
         size = mpGroups[group].size;
     }
 }
@@ -756,26 +767,19 @@ void KIconLoaderPrivate::normalizeIconMetadata(KIconLoader::Group &group, int &s
 QString KIconLoaderPrivate::makeCacheKey(const QString &name, KIconLoader::Group group,
                                          const QStringList &overlays, int size, int state) const
 {
-    // The KSharedDataCache is shared so add some namespacing.
-    QString key;
+    // The KSharedDataCache is shared so add some namespacing. The following code
+    // uses QStringBuilder (new in Qt 4.6)
 
-    if (group == KIconLoader::User) {
-        key = QLatin1String("$kicou_");
-    }
-    else {
-        key = QLatin1String("$kico_");
-    }
-
-    key.append(QString("%1_%2_%3").arg(name).arg(size).arg(overlays.join("_")));
-
-    if (group >= 0) {
-        key.append(mpEffect.fingerprint(group, state));
-    }
-    else {
-        key.append(QLatin1String("noeffect"));
-    }
-
-    return key;
+    return (group == KIconLoader::User
+                  ? QLatin1Literal("$kicou_")
+                  : QLatin1Literal("$kico_"))
+           % name
+           % QLatin1Char('_')
+           % QString::number(size)
+           % QLatin1Char('_')
+           % overlays.join("_")
+           % ( group >= 0 ? mpEffect.fingerprint(group, state)
+                          : *NULL_EFFECT_FINGERPRINT);
 }
 
 QImage KIconLoaderPrivate::createIconImage(const QString &path, int size)
@@ -796,6 +800,7 @@ QImage KIconLoaderPrivate::createIconImage(const QString &path, int size)
     }
     else
     {
+#ifndef _WIN32_WCE
         KSvgRenderer renderer(path, q);
 
         if (renderer.isValid()) {
@@ -804,6 +809,7 @@ QImage KIconLoaderPrivate::createIconImage(const QString &path, int size)
             QPainter p(&img);
             renderer.render(&p);
         }
+#endif
     }
 
     return img;
@@ -881,6 +887,13 @@ bool KIconLoaderPrivate::findCachedPixmapWithPath(const QString &key, QPixmap &d
             data = tempPixmap;
             path = tempPath;
 
+            // Since we're here we didn't have a QPixmap cache entry, add one now.
+            PixmapWithPath *newPixmapWithPath = new PixmapWithPath;
+            newPixmapWithPath->pixmap = data;
+            newPixmapWithPath->path = path;
+
+            mPixmapCache.insert(key, newPixmapWithPath, data.width() * data.height() + 1);
+
             return true;
         }
     }
@@ -1300,7 +1313,7 @@ QPixmap KIconLoader::loadIcon(const QString& _name, KIconLoader::Group group, in
     // Don't add the path to our unknown icon to the cache, only cache the
     // actual image.
     if (iconWasUnknown) {
-        icon.path = QString();
+        icon.path.clear();
     }
 
     d->insertCachedPixmapWithPath(key, pix, icon.path);
@@ -1577,6 +1590,7 @@ bool KIconLoader::alphaBlending(KIconLoader::Group group) const
 }
 
 // deprecated
+#ifndef KDE_NO_DEPRECATED
 QIcon KIconLoader::loadIconSet( const QString& name, KIconLoader::Group g, int s,
                                 bool canReturnNull )
 {
@@ -1590,6 +1604,7 @@ QIcon KIconLoader::loadIconSet( const QString& name, KIconLoader::Group g, int s
     iconset.addPixmap( tmp, QIcon::Normal, QIcon::On );
     return iconset;
 }
+#endif
 
 // Easy access functions
 
@@ -1600,11 +1615,13 @@ QPixmap DesktopIcon(const QString& name, int force_size, int state, const QStrin
 }
 
 // deprecated
+#ifndef KDE_NO_DEPRECATED
 QIcon DesktopIconSet(const QString& name, int force_size)
 {
     KIconLoader *loader = KIconLoader::global();
     return loader->loadIconSet(name, KIconLoader::Desktop, force_size);
 }
+#endif
 
 QPixmap BarIcon(const QString& name, int force_size, int state, const QStringList &overlays)
 {
@@ -1613,11 +1630,13 @@ QPixmap BarIcon(const QString& name, int force_size, int state, const QStringLis
 }
 
 // deprecated
+#ifndef KDE_NO_DEPRECATED
 QIcon BarIconSet(const QString& name, int force_size)
 {
     KIconLoader *loader = KIconLoader::global();
     return loader->loadIconSet( name, KIconLoader::Toolbar, force_size );
 }
+#endif
 
 QPixmap SmallIcon(const QString& name, int force_size, int state, const QStringList &overlays)
 {
@@ -1626,11 +1645,13 @@ QPixmap SmallIcon(const QString& name, int force_size, int state, const QStringL
 }
 
 // deprecated
+#ifndef KDE_NO_DEPRECATED
 QIcon SmallIconSet(const QString& name, int force_size)
 {
     KIconLoader *loader = KIconLoader::global();
     return loader->loadIconSet( name, KIconLoader::Small, force_size );
 }
+#endif
 
 QPixmap MainBarIcon(const QString& name, int force_size, int state, const QStringList &overlays)
 {
@@ -1639,11 +1660,13 @@ QPixmap MainBarIcon(const QString& name, int force_size, int state, const QStrin
 }
 
 // deprecated
+#ifndef KDE_NO_DEPRECATED
 QIcon MainBarIconSet(const QString& name, int force_size)
 {
     KIconLoader *loader = KIconLoader::global();
     return loader->loadIconSet( name, KIconLoader::MainToolbar, force_size );
 }
+#endif
 
 QPixmap UserIcon(const QString& name, int state, const QStringList &overlays)
 {
@@ -1652,11 +1675,13 @@ QPixmap UserIcon(const QString& name, int state, const QStringList &overlays)
 }
 
 // deprecated
+#ifndef KDE_NO_DEPRECATED
 QIcon UserIconSet(const QString& name)
 {
     KIconLoader *loader = KIconLoader::global();
     return loader->loadIconSet( name, KIconLoader::User );
 }
+#endif
 
 int IconSize(KIconLoader::Group group)
 {
