@@ -112,6 +112,11 @@ void KXmlGui_UnitTest::testFindVersionNumber_data()
     QTest::newRow("with a dot") << // as was found in autorefresh.rc
         "<?xml version = '1.0'?>\n"
         "<gui version = \"0.2\" name=\"foo\"/>\n" << QString() /*error*/;
+    QTest::newRow("with a comment") << // as found in kmail.rc
+        "<!DOCTYPE kpartgui>\n"
+        "<!-- This file should be synchronized with kmail_part.rc to provide\n"
+        "the same menu entries at the same place in KMail and Kontact  -->\n"
+        "<kpartgui version=\"452\" name=\"kmmainwin\">\n" << "452";
 }
 
 void KXmlGui_UnitTest::testFindVersionNumber()
@@ -312,7 +317,7 @@ void KXmlGui_UnitTest::testPartMerging()
                              << "host_after_merge" << "host_after_merge_2" << "last_from_host"
                              << "file_new" << "file_open" << "file_quit");
     hostClient.createGUI(hostXml, true /*ui_standards.rc*/);
-    QMainWindow mainWindow;
+    KMainWindow mainWindow;
     KXMLGUIBuilder builder(&mainWindow);
     KXMLGUIFactory factory(&builder);
     factory.addClient(&hostClient);
@@ -402,6 +407,58 @@ void KXmlGui_UnitTest::testPartMerging()
                  << "separator"
                  << "file_quit"
                  << "other_file_action");
+    factory.removeClient(&partClient);
+    factory.removeClient(&hostClient);
+}
+
+void KXmlGui_UnitTest::testPartMergingSettings() // #252911
+{
+    const QByteArray hostXml =
+        "<?xml version = '1.0'?>\n"
+        "<!DOCTYPE gui SYSTEM \"kpartgui.dtd\">\n"
+        "<gui version=\"1\" name=\"foo\" >\n"
+        "<MenuBar>\n"
+// The solution was to remove the duplicated definition
+//        " <Menu name=\"settings\"><text>&amp;Settings</text>\n"
+//        "    <Action name=\"options_configure_keybinding\"/>\n"
+//        "    <Action name=\"options_configure_toolbars\"/>\n"
+//        "    <Merge name=\"configure_merge\"/>\n"
+//        "    <Separator/>\n"
+//        "    <Merge/>\n"
+//        " </Menu>\n"
+        "</MenuBar></gui>\n";
+    TestGuiClient hostClient;
+    hostClient.createActions(QStringList() << "options_configure_keybinding" << "options_configure_toolbars");
+    hostClient.createGUI(hostXml, true /*ui_standards.rc*/);
+    //kDebug() << hostClient.domDocument().toString();
+    KMainWindow mainWindow;
+    KXMLGUIBuilder builder(&mainWindow);
+    KXMLGUIFactory factory(&builder);
+    factory.addClient(&hostClient);
+    QWidget* settingsMenu = qobject_cast<QMenu *>(factory.container("settings", &hostClient));
+    QVERIFY(settingsMenu);
+    //debugActions(settingsMenu->actions());
+
+    const QByteArray partXml =
+        "<?xml version = '1.0'?>\n"
+        "<!DOCTYPE gui SYSTEM \"kpartgui.dtd\">\n"
+        "<gui version=\"1\" name=\"foo\" >\n"
+        "<MenuBar>\n"
+        " <Menu name=\"settings\"><text>&amp;Settings</text>\n"
+        "    <Action name=\"configure_klinkstatus\"/>\n"
+        " </Menu>\n"
+        "</MenuBar></gui>\n";
+    TestGuiClient partClient(partXml);
+    partClient.createActions(QStringList() << "configure_klinkstatus");
+    factory.addClient(&partClient);
+    //debugActions(settingsMenu->actions());
+    checkActions(settingsMenu->actions(), QStringList()
+                 << "separator" // that's ok, QMenuPrivate::filterActions won't show it
+                 << "options_configure_keybinding"
+                 << "options_configure_toolbars"
+                 << "configure_klinkstatus");
+    factory.removeClient(&partClient);
+    factory.removeClient(&hostClient);
 }
 
 void KXmlGui_UnitTest::testUiStandardsMerging_data()
@@ -511,7 +568,7 @@ void KXmlGui_UnitTest::testUiStandardsMerging()
     QCOMPARE(docElem.attribute("name"), QString("foo")); // not standard_containers from ui_standards.rc
     QCOMPARE(docElem.attribute("version"), QString("1")); // not standard_containers from ui_standards.rc
 
-    QMainWindow mainWindow;
+    KMainWindow mainWindow;
     KXMLGUIBuilder builder(&mainWindow);
     KXMLGUIFactory factory(&builder);
     factory.addClient(&client);
@@ -519,6 +576,8 @@ void KXmlGui_UnitTest::testUiStandardsMerging()
     const QStringList containerNames = collectMenuNames(factory);
     //kDebug() << containerNames;
     QCOMPARE(containerNames, expectedMenus);
+
+    factory.removeClient(&client);
 }
 
 void KXmlGui_UnitTest::testActionListAndSeparator()
@@ -537,8 +596,8 @@ void KXmlGui_UnitTest::testActionListAndSeparator()
         "</gui>";
 
     TestGuiClient client(xml);
-    client.createActions(QStringList() << "view_add_to_new_group" << "action1");
-    QMainWindow mainWindow;
+    client.createActions(QStringList() << "view_add_to_new_group");
+    KMainWindow mainWindow;
     KXMLGUIBuilder builder(&mainWindow);
     KXMLGUIFactory factory(&builder);
     factory.addClient(&client);
@@ -555,9 +614,13 @@ void KXmlGui_UnitTest::testActionListAndSeparator()
 
     kDebug() << "Now plugging the actionlist";
 
+    KAction* action1 = new KAction(this);
+    action1->setObjectName("action1");
+    action1->setShortcut(KShortcut("Ctrl+2"));
     QList<QAction*> actionList;
-    actionList << client.actionCollection()->action("action1");
+    actionList << action1;
     client.plugActionList("view_groups_list", actionList);
+    QCOMPARE(action1->shortcut().toString(), QString("Ctrl+2"));
 
     //debugActions(menu->actions());
     checkActions(menu->actions(), QStringList()
@@ -585,6 +648,7 @@ void KXmlGui_UnitTest::testActionListAndSeparator()
                  << "action1"
                  << "separator"
                  << "view_add_to_new_group");
+    factory.removeClient(&client);
 }
 
 void KXmlGui_UnitTest::testHiddenToolBar()
@@ -703,7 +767,7 @@ void KXmlGui_UnitTest::testAutoSaveSettings()
     }
 }
 
-void KXmlGui_UnitTest::testDeletedContainers()
+void KXmlGui_UnitTest::testDeletedContainers() // deleted="true"
 {
     const QByteArray xml =
         "<?xml version = '1.0'?>\n"
@@ -762,10 +826,8 @@ void KXmlGui_UnitTest::testTopLevelSeparator() {
         " <Menu name=\"after_separator\"><text>After Separator</text></Menu>\n"
         "</MenuBar>\n"
         "</gui>";
-    
+
     TestXmlGuiWindow mainWindow(xml);
-    mainWindow.setAutoSaveSettings(false);
-    mainWindow.createActions(QStringList());
     mainWindow.createGUI();
 
     checkActions(mainWindow.menuBar()->actions(), QStringList()
@@ -776,8 +838,10 @@ void KXmlGui_UnitTest::testTopLevelSeparator() {
                  << "help");
 }
 
-void KXmlGui_UnitTest::testMenuNames() {
-    const QByteArray xml = 
+// Check that the objectName() of the menus is set from the name in the XML file
+void KXmlGui_UnitTest::testMenuNames()
+{
+    const QByteArray xml =
         "<?xml version = '1.0'?>\n"
         "<!DOCTYPE gui SYSTEM \"kpartgui.dtd\">\n"
         "<gui version=\"1\" name=\"foo\" >\n"
@@ -785,10 +849,8 @@ void KXmlGui_UnitTest::testMenuNames() {
         " <Menu name=\"filemenu\"><text>File Menu</text></Menu>\n"
         "</MenuBar>\n"
         "</gui>";
-    
+
     TestXmlGuiWindow mainWindow(xml);
-    mainWindow.setAutoSaveSettings(false);
-    mainWindow.createActions(QStringList());
     mainWindow.createGUI();
 
     checkActions(mainWindow.menuBar()->actions(), QStringList()
@@ -797,6 +859,17 @@ void KXmlGui_UnitTest::testMenuNames() {
                  << "help" );
 }
 
+// Test what happens when the application's rc file isn't found
+// We want a warning to be printed, but we don't want to see all menus from ui_standards.rc
+void KXmlGui_UnitTest::testMenusNoXmlFile()
+{
+    TestXmlGuiWindow mainWindow;
+    mainWindow.createGUIBad();
+
+    checkActions(mainWindow.menuBar()->actions(), QStringList()
+                 << "help" );
+}
+
 void KXmlGui_UnitTest::testXMLFileReplacement() {
     // to differentiate "original" and replacement xml file, one is created with "modified" toolbars
     KTemporaryFile fileOrig;
@@ -840,26 +913,55 @@ void KXmlGui_UnitTest::testXMLFileReplacement() {
     QVERIFY(!reloadedXml.contains("<Action name=\"print\""));
     QVERIFY(reloadedXml.contains("<Action name=\"home\""));
     QVERIFY(reloadedXml.contains("<ActionProperties>"));
+
+    // Check what happens when the local file doesn't exist
+    TestGuiClient client2;
+    QFile::remove(filenameLocal);
+    client2.replaceXMLFile(filenameReplace, filenameLocal);
+    xml = client2.domDocument().toString();
+    //qDebug() << xml;
+    QVERIFY(!xml.contains("<Action name=\"print\""));
+    QVERIFY(xml.contains("<Action name=\"home\"")); // modified toolbars
+    QVERIFY(!xml.contains("<ActionProperties>")); // but no local xml file
 }
 
 void KXmlGui_UnitTest::testClientDestruction() { // #170806
-    const QByteArray xml = 
+    const QByteArray hostXml =
         "<?xml version = '1.0'?>\n"
         "<!DOCTYPE gui SYSTEM \"kpartgui.dtd\">\n"
         "<gui version=\"1\" name=\"foo\" >\n"
         "<MenuBar>\n"
-        " <Menu name=\"filemenu\"><text>File Menu</text></Menu>\n"
+        " <Menu name=\"file\"><text>&amp;File</text>\n"
+        " </Menu>\n"
+        " <Merge/>\n"
+        "</MenuBar>\n"
+        "</gui>";
+    const QByteArray xml = "<?xml version = '1.0'?>\n"
+        "<!DOCTYPE gui SYSTEM \"kpartgui.dtd\">\n"
+        "<gui version=\"1\" name=\"foo\" >\n"
+        "<MenuBar>\n"
+        " <Menu name=\"file\"><text>&amp;File</text>\n"
+        "  <Action name=\"file_open\"/>\n"
+        "  <Action name=\"file_quit\"/>\n"
+        " </Menu>\n"
         "</MenuBar>\n"
         "</gui>";
 
-    TestXmlGuiWindow mainWindow(xml);
+    TestXmlGuiWindow mainWindow(hostXml);
     TestGuiClient* client = new TestGuiClient(xml);
+    client->createActions(QStringList() << "file_open" << "file_quit");
     mainWindow.insertChildClient(client);
-    mainWindow.setAutoSaveSettings(false);
     mainWindow.createGUI();
 
+    checkActions(mainWindow.menuBar()->actions(), QStringList()
+                 << "file" << "separator" << "help" );
+
     QVERIFY(mainWindow.factory()->clients().contains(client));
     delete client;
     QVERIFY(!mainWindow.factory()->clients().contains(client));
+
+    // No change, because deletion is fast, it doesn't do manual unplugging.
+    checkActions(mainWindow.menuBar()->actions(), QStringList()
+                 << "file" << "separator" << "help" );
 }
 
