@@ -32,23 +32,25 @@
 #include "modelspy.h"
 #include "persistentchangelist.h"
 
+#include "proxymodeltestsuite_export.h"
+
 typedef QList<QVariantList> SignalList;
 
 Q_DECLARE_METATYPE( SignalList )
 
-enum Persistence
+enum PROXYMODELTESTSUITE_EXPORT Persistence
 {
   LazyPersistence,
   ImmediatePersistence
 };
 
-enum SourceModel
+enum PROXYMODELTESTSUITE_EXPORT SourceModel
 {
   DynamicTree,
   IntermediateProxy
 };
 
-class BuiltinTestDataInterface
+class PROXYMODELTESTSUITE_EXPORT BuiltinTestDataInterface
 {
 public:
   virtual ~BuiltinTestDataInterface() { }
@@ -72,7 +74,7 @@ private:
 
 };
 
-class BuiltinTestInterface : BuiltinTestDataInterface
+class PROXYMODELTESTSUITE_EXPORT BuiltinTestInterface : BuiltinTestDataInterface
 {
 public:
   virtual ~BuiltinTestInterface() { }
@@ -118,7 +120,7 @@ private:
 
 };
 
-class ProxyModelTest : public QObject, protected BuiltinTestInterface
+class PROXYMODELTESTSUITE_EXPORT ProxyModelTest : public QObject, protected BuiltinTestInterface
 {
   Q_OBJECT
 public:
@@ -247,7 +249,7 @@ private:
   QString m_currentTest;
 };
 
-class ProxyModelTestData : public QObject, BuiltinTestDataInterface
+class PROXYMODELTESTSUITE_EXPORT ProxyModelTestData : public QObject, BuiltinTestDataInterface
 {
   Q_OBJECT
 public:
@@ -268,6 +270,16 @@ protected:
     QSKIP("Test not implemented", SkipSingle);
   }
 
+  void skipTestData(const QString &name)
+  {
+    processTestName(name);
+
+    QTest::addColumn<SignalList>("signalList");
+    QTest::addColumn<PersistentChangeList>("changeList");
+
+    QTest::newRow(name.toAscii()) << (SignalList() << (QVariantList() << "skip") ) << PersistentChangeList();
+  }
+
   void processTestName(const QString &name)
   {
     if (m_currentTestFunction != QTest::currentTestFunction())
@@ -590,6 +602,8 @@ protected:
   QSet<QString> m_testNames;
 };
 
+PROXYMODELTESTSUITE_EXPORT uint qHash( const QVariant & var );
+
 #define PROXYMODELTEST(TestData, TemplateArg, IntermediateProxy, LazyPersistence, Config) \
   if (testObjects.isEmpty() || testObjects.contains(testNum)) { \
     proxyModelTestClass->setTestData(new TestData<TemplateArg>(proxyModelTestClass)); \
@@ -640,6 +654,8 @@ protected:
     foreach(const QString &arg, app.arguments()) \
     { \
       int testObject = arg.toInt(&ok); \
+      if (arg == "-count") \
+        continue; \
       if (!ok) \
       { \
         arguments.append(arg); \
