@@ -41,7 +41,6 @@ public:
     QWidget *m_parent; // necessary to set the focus back
     QString cancelText;
     bool tabHandling : 1;
-    bool down_workaround : 1; // next call to down() selects the first item
     bool upwardBox : 1;
     bool emitSelected : 1;
 };
@@ -51,7 +50,6 @@ KCompletionBox::KCompletionBox( QWidget *parent )
 {
     d->m_parent        = parent;
     d->tabHandling     = true;
-    d->down_workaround = false;
     d->upwardBox       = false;
     d->emitSelected    = true;
 
@@ -65,14 +63,6 @@ KCompletionBox::KCompletionBox( QWidget *parent )
 
     connect( this, SIGNAL( itemDoubleClicked( QListWidgetItem * )),
              SLOT( slotActivated( QListWidgetItem * )) );
-
-#ifdef __GNUC__
-#warning "Check if this workaround can be removed in KDE 4"
-#endif
-
-    // grmbl, just QListBox workarounds :[ Thanks Volker.
-    connect( this, SIGNAL( currentItemChanged( QListWidgetItem * , QListWidgetItem * )),
-             SLOT( slotCurrentChanged() ));
     connect( this, SIGNAL( itemClicked( QListWidgetItem * )),
              SLOT( slotItemClicked( QListWidgetItem * )) );
 }
@@ -273,10 +263,9 @@ void KCompletionBox::popup()
     if ( count() == 0 )
         hide();
     else {
-        //TODO KDE 4 - Port: ensureCurrentVisible();
         bool block = signalsBlocked();
         blockSignals( true );
-        setCurrentItem( 0 );
+        setCurrentRow( -1 );
         blockSignals( block );
         clearSelection();
         if ( !isVisible() )
@@ -296,9 +285,9 @@ void KCompletionBox::sizeAndPosition()
     int x = currentPos.x(), y = currentPos.y();
     if ( d->m_parent ) {
       if ( !isVisible() ) {
-        QRect screenSize = KGlobalSettings::desktopGeometry(d->m_parent);
-
         QPoint orig = globalPositionHint();
+        QRect screenSize = KGlobalSettings::desktopGeometry(orig);
+
         x = orig.x() + geom.x();
         y = orig.y() + geom.y();
 
@@ -414,23 +403,18 @@ QSize KCompletionBox::sizeHint() const
 
 void KCompletionBox::down()
 {
-    int i = currentRow();
-
-    if ( i == 0 && d->down_workaround ) {
-        d->down_workaround = false;
-        setCurrentRow( 0 );
-        item(0)->setSelected(true);
-        emit currentTextChanged( currentItem()->text() );
+    const int i = currentRow();
+    if (i < count() - 1) {
+        setCurrentRow(i + 1);
     }
-
-    else if ( i < (int) count() - 1 )
-        setCurrentRow( i + 1 );
 }
 
 void KCompletionBox::up()
 {
-    if ( currentItem() && row(currentItem()) > 0 )
-        setCurrentItem( item(row(currentItem()) - 1) );
+    const int i = currentRow();
+    if (i > 0) {
+        setCurrentRow(i - 1);
+    }
 }
 
 void KCompletionBox::pageDown()
@@ -452,7 +436,7 @@ void KCompletionBox::pageUp()
 
 void KCompletionBox::home()
 {
-    setCurrentItem( 0 );
+    setCurrentRow( 0 );
 }
 
 void KCompletionBox::end()
@@ -508,7 +492,7 @@ void KCompletionBox::insertItems( const QStringList& items, int index )
     blockSignals( true );
     KListWidget::insertItems( index, items );
     blockSignals( block );
-    d->down_workaround = true;
+    setCurrentRow(-1);
 }
 
 void KCompletionBox::setItems( const QStringList& items )
@@ -562,25 +546,12 @@ void KCompletionBox::setItems( const QStringList& items )
         sizeAndPosition();
 
     blockSignals(block);
-    d->down_workaround = true;
-}
-
-void KCompletionBox::slotCurrentChanged()
-{
-    if (currentItem())
-      emit currentTextChanged(currentItem()->text());
-    d->down_workaround = false;
 }
 
 void KCompletionBox::slotItemClicked( QListWidgetItem *item )
 {
     if ( item )
     {
-        if ( d->down_workaround ) {
-            d->down_workaround = false;
-            emit currentTextChanged( item->text() );
-        }
-
         hide();
         emit currentTextChanged( item->text() );
         emit activated( item->text() );
