@@ -313,9 +313,9 @@ UString WrapScriptableObject::toString(ExecState*) const
     }
 
     if (type == FunctionRef) {
-        return "[function ImportedScriptable:" + iface + "/" + field + "]";
+        return QString(QLatin1String("[function ImportedScriptable:") + iface + QLatin1Char('/') + field + QLatin1Char(']'));
     } else {
-        return "[object ImportedScriptable:" + iface + "]";
+        return QString(QLatin1String("[object ImportedScriptable:") + iface + QLatin1Char(']'));
     }
 }
 
@@ -834,7 +834,7 @@ QVariant KHTMLPartScriptable::encloserForKid(KParts::ScriptableExtension* kid)
     // ### this should never actually get an iframe once iframes are fixed
     if (!f->m_partContainerElement.isNull()) {
         return ScriptableOperations::exportValue(
-                getDOMNode(i->globalExec(), f->m_partContainerElement), true);
+                getDOMNode(i->globalExec(), f->m_partContainerElement.data()), true);
     }
 
     kWarning(6031) << "could not find the part container";
