@@ -23,7 +23,6 @@
 #include <httpfilter/httpfilter.h>
 
 #include <kparts/part.h>
-#include <kparts/factory.h>
 #include <kparts/browserextension.h>
 #include <kaboutdata.h>
 #include <QtCore/QDate>
@@ -39,7 +38,7 @@ class KMultiPart : public KParts::ReadOnlyPart
     Q_OBJECT
 public:
     KMultiPart( QWidget *parentWidget,
-                QObject *parent, const QStringList& );
+                QObject *parent, const QVariantList& );
     virtual ~KMultiPart();
 
     virtual bool openFile() { return false; }
