@@ -21,24 +21,27 @@
 */
 
 #include "kbookmarkmanager.h"
+
+#include <QtCore/QFile>
+#include <QtCore/QFileInfo>
+#include <QtCore/QProcess>
+#include <QtCore/QRegExp>
+#include <QtCore/QTextStream>
+#include <QtDBus/QtDBus>
+#include <QtGui/QApplication>
+
+#include <kconfiggroup.h>
+#include <kdebug.h>
+#include <kdirwatch.h>
+#include <klocale.h>
+#include <kmessagebox.h>
+#include <ksavefile.h>
+#include <kstandarddirs.h>
+
 #include "kbookmarkmenu.h"
 #include "kbookmarkmenu_p.h"
 #include "kbookmarkimporter.h"
 #include "kbookmarkdialog.h"
-#include <kdebug.h>
-#include <kstandarddirs.h>
-#include <ksavefile.h>
-#include <qregexp.h>
-#include <kmessagebox.h>
-#include <qprocess.h>
-#include <klocale.h>
-#include <kdirwatch.h>
-#include <QtGui/QApplication>
-#include <kconfiggroup.h>
-#include <qfile.h>
-#include <qfileinfo.h>
-#include <QtDBus/QtDBus>
-#include <QtCore/QTextStream>
 #include "kbookmarkmanageradaptor_p.h"
 
 #define BOOKMARK_CHANGE_NOTIFY_INTERFACE "org.kde.KIO.KBookmarkManager"
@@ -49,16 +52,19 @@ public:
     ~KBookmarkManagerList() {
         qDeleteAll( begin() , end() ); // auto-delete functionality
     }
+
+    QReadWriteLock lock;
 };
 
 K_GLOBAL_STATIC(KBookmarkManagerList, s_pSelf)
 
 class KBookmarkMap : private KBookmarkGroupTraverser {
 public:
-    KBookmarkMap( KBookmarkManager * );
-    void update();
+    KBookmarkMap() : m_mapNeedsUpdate(true) {}
+    void setNeedsUpdate() { m_mapNeedsUpdate = true; }
+    void update(KBookmarkManager*);
     QList<KBookmark> find( const QString &url ) const
-    { return m_bk_map[url]; }
+    { return m_bk_map.value(url); }
 private:
     virtual void visit(const KBookmark &);
     virtual void visitEnter(const KBookmarkGroup &) { ; }
@@ -66,35 +72,18 @@ private:
 private:
     typedef QList<KBookmark> KBookmarkList;
     QMap<QString, KBookmarkList> m_bk_map;
-    KBookmarkManager *m_manager;
+    bool m_mapNeedsUpdate;
 };
 
-class KBookmarkMapStatic
+void KBookmarkMap::update(KBookmarkManager *manager)
 {
-public:
-    KBookmarkMapStatic()
-        : map( 0 )
-    {
+    if (m_mapNeedsUpdate) {
+        m_mapNeedsUpdate = false;
+
+        m_bk_map.clear();
+        KBookmarkGroup root = manager->root();
+        traverse(root);
     }
-    ~KBookmarkMapStatic()
-    {
-        delete map;
-    }
-
-    KBookmarkMap *map;
-};
-
-K_GLOBAL_STATIC(KBookmarkMapStatic, s_bk)
-
-KBookmarkMap::KBookmarkMap( KBookmarkManager *manager ) {
-    m_manager = manager;
-}
-
-void KBookmarkMap::update()
-{
-    m_bk_map.clear();
-    KBookmarkGroup root = m_manager->root();
-    traverse(root);
 }
 
 void KBookmarkMap::visit(const KBookmark &bk)
@@ -115,28 +104,33 @@ public:
       , m_dbusObjectName(dbusObjectName)
       , m_docIsLoaded(bDocIsloaded)
       , m_update(false)
+      , m_dialogAllowed(true)
+      , m_dialogParent(0)
+      , m_browserEditor(false)
       , m_typeExternal(false)
       , m_kDirWatch(0)
-
     {}
 
     ~Private() {
         delete m_kDirWatch;
-    }    
-    
+    }
+
     mutable QDomDocument m_doc;
     mutable QDomDocument m_toolbarDoc;
     QString m_bookmarksFile;
     QString m_dbusObjectName;
     mutable bool m_docIsLoaded;
     bool m_update;
+    bool m_dialogAllowed;
+    QWidget *m_dialogParent;
 
     bool m_browserEditor;
     QString m_editorCaption;
-    
+
     bool m_typeExternal;
     KDirWatch * m_kDirWatch;  // for external bookmark files
 
+    KBookmarkMap m_map;
 };
 
 // ################
@@ -149,15 +143,27 @@ static KBookmarkManager* lookupExisting(const QString& bookmarksFile)
         if ( (*bmit)->path() == bookmarksFile )
             return *bmit;
     }
-    return 0; 
+    return 0;
 }
 
 
 KBookmarkManager* KBookmarkManager::managerForFile( const QString& bookmarksFile, const QString& dbusObjectName )
 {
-    KBookmarkManager* mgr = lookupExisting(bookmarksFile);
-    if (mgr) return mgr;
-    
+    KBookmarkManager* mgr(0);
+    {
+        QReadLocker readLock(&s_pSelf->lock);
+        mgr = lookupExisting(bookmarksFile);
+        if (mgr) {
+            return mgr;
+        }
+    }
+
+    QWriteLocker writeLock(&s_pSelf->lock);
+    mgr = lookupExisting(bookmarksFile);
+    if (mgr) {
+        return mgr;
+    }
+
     mgr = new KBookmarkManager( bookmarksFile, dbusObjectName );
     s_pSelf->append( mgr );
     return mgr;
@@ -165,9 +171,21 @@ KBookmarkManager* KBookmarkManager::managerForFile( const QString& bookmarksFile
 
 KBookmarkManager* KBookmarkManager::managerForExternalFile( const QString& bookmarksFile )
 {
-    KBookmarkManager* mgr = lookupExisting(bookmarksFile);
-    if (mgr) return mgr;
-    
+    KBookmarkManager* mgr(0);
+    {
+        QReadLocker readLock(&s_pSelf->lock);
+        mgr = lookupExisting(bookmarksFile);
+        if (mgr) {
+            return mgr;
+        }
+    }
+
+    QWriteLocker writeLock(&s_pSelf->lock);
+    mgr = lookupExisting(bookmarksFile);
+    if (mgr) {
+        return mgr;
+    }
+
     mgr = new KBookmarkManager( bookmarksFile );
     s_pSelf->append( mgr );
     return mgr;
@@ -200,7 +218,7 @@ KBookmarkManager::KBookmarkManager( const QString & bookmarksFile, const QString
 {
     if(dbusObjectName.isNull()) // get dbusObjectName from file
         if ( QFile::exists(d->m_bookmarksFile) )
-            parse(); //sets d->m_dbusObjectnName
+            parse(); //sets d->m_dbusObjectName
 
     init( "/KBookmarkManager/"+d->m_dbusObjectName );
 
@@ -247,7 +265,7 @@ KBookmarkManager::KBookmarkManager(const QString & bookmarksFile)
     QObject::connect( d->m_kDirWatch, SIGNAL(deleted(const QString&)),
             this, SLOT(slotFileChanged(const QString&)));
     kDebug(7043) << "starting KDirWatch for " << d->m_bookmarksFile;
-}  
+}
 
 KBookmarkManager::KBookmarkManager( )
     : d(new Private(true))
@@ -283,17 +301,30 @@ void KBookmarkManager::slotFileChanged(const QString& path)
         parse();
         // Tell our GUI
         // (emit where group is "" to directly mark the root menu as dirty)
-        emit changed( "", QString() );      
+        emit changed( "", QString() );
     }
 }
 
 KBookmarkManager::~KBookmarkManager()
 {
-    if(!s_pSelf.isDestroyed())
-        s_pSelf->removeAll( this );
+    if (!s_pSelf.isDestroyed()) {
+        s_pSelf->removeAll(this);
+    }
+
     delete d;
 }
 
+bool KBookmarkManager::autoErrorHandlingEnabled() const
+{
+    return d->m_dialogAllowed;
+}
+
+void KBookmarkManager::setAutoErrorHandlingEnabled( bool enable, QWidget *parent )
+{
+    d->m_dialogAllowed = enable;
+    d->m_dialogParent = parent;
+}
+
 void KBookmarkManager::setUpdate( bool update )
 {
     d->m_update = update;
@@ -358,9 +389,8 @@ void KBookmarkManager::parse() const
     d->m_doc.insertBefore( pi, docElem );
 
     file.close();
-    if ( !s_bk->map )
-        s_bk->map = new KBookmarkMap( const_cast<KBookmarkManager*>( this ) );
-    s_bk->map->update();
+
+    d->m_map.setNeedsUpdate();
 }
 
 bool KBookmarkManager::save( bool toolbarCache ) const
@@ -410,15 +440,17 @@ bool KBookmarkManager::saveAs( const QString & filename, bool toolbarCache ) con
     static int hadSaveError = false;
     file.abort();
     if ( !hadSaveError ) {
-        QString error = i18n("Unable to save bookmarks in %1. Reported error was: %2. "
+        QString err = i18n("Unable to save bookmarks in %1. Reported error was: %2. "
                              "This error message will only be shown once. The cause "
                              "of the error needs to be fixed as quickly as possible, "
                              "which is most likely a full hard drive.",
                          filename, file.errorString());
-        if (qApp->type() != QApplication::Tty)
-            KMessageBox::error( QApplication::activeWindow(), error );
-        else
-            kError() << error << endl;
+
+        if (d->m_dialogAllowed && qApp->type() != QApplication::Tty && QThread::currentThread() == qApp->thread())
+            KMessageBox::error( QApplication::activeWindow(), err );
+
+        kError() << QString("Unable to save bookmarks in %1. File reported the following error-code: %2.").arg(filename).arg(file.error());
+        emit const_cast<KBookmarkManager*>(this)->error(err);
     }
     hadSaveError = true;
     return false;
@@ -604,12 +636,8 @@ void KBookmarkManager::slotEditBookmarksAtAddress( const QString& address )
 ///////
 bool KBookmarkManager::updateAccessMetadata( const QString & url )
 {
-    if (!s_bk->map) {
-        s_bk->map = new KBookmarkMap(this);
-        s_bk->map->update();
-    }
-
-    QList<KBookmark> list = s_bk->map->find(url);
+    d->m_map.update(this);
+    QList<KBookmark> list = d->m_map.find(url);
     if ( list.count() == 0 )
         return false;
 
@@ -622,14 +650,8 @@ bool KBookmarkManager::updateAccessMetadata( const QString & url )
 
 void KBookmarkManager::updateFavicon( const QString &url, const QString &faviconurl )
 {
-    Q_UNUSED(faviconurl);
-
-    if (!s_bk->map) {
-        s_bk->map = new KBookmarkMap(this);
-        s_bk->map->update();
-    }
-
-    QList<KBookmark> list = s_bk->map->find(url);
+    d->m_map.update(this);
+    QList<KBookmark> list = d->m_map.find(url);
     for ( QList<KBookmark>::iterator it = list.begin();
           it != list.end(); ++it )
     {
