@@ -42,6 +42,8 @@ namespace Nepomuk {
             QueryPrivate()
                 : m_limit( 0 ),
                   m_offset( 0 ),
+                  m_fullTextScoringEnabled( false ),
+                  m_fullTextScoringSortOrder( Qt::DescendingOrder ),
                   m_isFileQuery( false ),
                   m_fileMode( FileQuery::QueryFilesAndFolders ) {
             }
@@ -50,19 +52,18 @@ namespace Nepomuk {
             QStringList buildRequestPropertyVariableList() const;
             QString buildRequestPropertyPatterns() const;
 
-            /**
-             * Removes redundant AND and OR operators.
-             */
-            static Term optimizeTerm( const Term& term );
-
             Term m_term;
             int m_limit;
             int m_offset;
 
+            bool m_fullTextScoringEnabled;
+            Qt::SortOrder m_fullTextScoringSortOrder;
+            Query::QueryFlags m_flags;
+
             QList<Query::RequestProperty> m_requestProperties;
 
             bool m_isFileQuery;
-            KUrl::List m_includeFolders;
+            QHash<KUrl, bool> m_includeFolders;
             KUrl::List m_excludeFolders;
             FileQuery::FileMode m_fileMode;
         };
