@@ -19,18 +19,25 @@
 
 #include "packagestructure.h"
 
+#include "config-plasma.h"
+
 #include <QDir>
 #include <QMap>
 #include <QFileInfo>
 
 #include <kconfiggroup.h>
+#include <kdebug.h>
+#ifndef PLASMA_NO_KIO
+#include <kio/job.h>
+#endif
+#include <kmimetype.h>
 #include <kstandarddirs.h>
 #include <kservicetypetrader.h>
-#include <kurl.h>
+#include <ktar.h>
 #include <ktemporaryfile.h>
 #include <ktempdir.h>
+#include <kurl.h>
 #include <kzip.h>
-#include <kio/job.h>
 
 #include "package.h"
 #include "private/packages_p.h"
@@ -50,14 +57,14 @@ class ContentStructure
 
         ContentStructure(const ContentStructure &other)
         {
-            path = other.path;
+            paths = other.paths;
             name = other.name;
             mimetypes = other.mimetypes;
             directory = other.directory;
             required = other.required;
         }
 
-        QString path;
+        QStringList paths;
         QString name;
         QStringList mimetypes;
         bool directory : 1;
@@ -69,12 +76,12 @@ class PackageStructurePrivate
 public:
     PackageStructurePrivate(const QString &t)
         : type(t),
-          contentsPrefix("contents/"),
           packageRoot("plasma/plasmoids"),
           servicePrefix("plasma-applet-"),
           metadata(0),
           externalPaths(false)
     {
+        contentsPrefixPaths << "contents/";
     }
 
     ~PackageStructurePrivate()
@@ -83,12 +90,13 @@ public:
     }
 
     void createPackageMetadata(const QString &path);
+    QStringList entryList(const QString &prefix, const QString &requestedPath);
 
     static QHash<QString, PackageStructure::Ptr> structures;
 
     QString type;
     QString path;
-    QString contentsPrefix;
+    QStringList contentsPrefixPaths;
     QString packageRoot;
     QString servicePrefix;
     QMap<QByteArray, ContentStructure> contents;
@@ -181,7 +189,9 @@ PackageStructure::Ptr PackageStructure::load(const QString &packageFormat)
         KConfig config(url.toLocalFile(), KConfig::SimpleConfig);
         structure->read(&config);
         PackageStructurePrivate::structures[structure->type()] = structure;
-    } else {
+    }
+#ifndef PLASMA_NO_KIO
+    else {
         KTemporaryFile tmp;
         if (tmp.open()) {
             KIO::Job *job = KIO::file_copy(url, KUrl(tmp.fileName()),
@@ -193,6 +203,7 @@ PackageStructure::Ptr PackageStructure::load(const QString &packageFormat)
             }
         }
     }
+#endif
 
     return structure;
 }
@@ -273,19 +284,32 @@ QStringList PackageStructure::entryList(const char *key)
         return QStringList();
     }
 
-    QDir dir(d->path + d->contentsPrefix + p);
-
-    if (dir.exists()) {
-        if (d->externalPaths) {
-            return dir.entryList(QDir::Files | QDir::Readable);
+    QStringList list;
+    if (d->contentsPrefixPaths.isEmpty()) {
+        // no prefixes is the same as d->contentsPrefixPths with QStringList() << QString()
+        list << d->entryList(QString(), p);
+    } else {
+        foreach (QString prefix, d->contentsPrefixPaths) {
+            list << d->entryList(prefix, p);
         }
+    }
 
-        // ensure that we don't return files outside of our base path
-        // due to symlink or ../ games
-        QString canonicalized = dir.canonicalPath();
-        if (canonicalized.startsWith(d->path)) {
-            return dir.entryList(QDir::Files | QDir::Readable);
-        }
+    return list;
+}
+
+QStringList PackageStructurePrivate::entryList(const QString &prefix, const QString &requestedPath)
+{
+    QDir dir(path + prefix + requestedPath);
+
+    if (externalPaths) {
+        return dir.entryList(QDir::Files | QDir::Readable);
+    }
+
+    // ensure that we don't return files outside of our base path
+    // due to symlink or ../ games
+    QString canonicalized = dir.canonicalPath();
+    if (canonicalized.startsWith(path)) {
+        return dir.entryList(QDir::Files | QDir::Readable);
     }
 
     return QStringList();
@@ -295,8 +319,16 @@ void PackageStructure::addDirectoryDefinition(const char *key,
                                               const QString &path, const QString &name)
 {
     ContentStructure s;
-    s.name = name;
-    s.path = path;
+
+    if (d->contents.contains(key)) {
+        s = d->contents[key];
+    }
+
+    if (!name.isEmpty()) {
+        s.name = name;
+    }
+
+    s.paths.append(path);
     s.directory = true;
 
     d->contents[key] = s;
@@ -305,13 +337,26 @@ void PackageStructure::addDirectoryDefinition(const char *key,
 void PackageStructure::addFileDefinition(const char *key, const QString &path, const QString &name)
 {
     ContentStructure s;
-    s.name = name;
-    s.path = path;
+
+    if (d->contents.contains(key)) {
+        s = d->contents[key];
+    }
+
+    if (!name.isEmpty()) {
+        s.name = name;
+    }
+
+    s.paths.append(path);
     s.directory = false;
 
     d->contents[key] = s;
 }
 
+void PackageStructure::removeDefinition(const char *key)
+{
+    d->contents.remove(key);
+}
+
 QString PackageStructure::path(const char *key) const
 {
     //kDebug() << "looking for" << key;
@@ -320,8 +365,20 @@ QString PackageStructure::path(const char *key) const
         return QString();
     }
 
-    //kDebug() << "found" << key << "and the value is" << it.value().path;
-    return it.value().path;
+    //kDebug() << "found" << key << "and the value is" << it.value().paths.first();
+    return it.value().paths.first();
+}
+
+QStringList PackageStructure::searchPath(const char *key) const
+{
+    //kDebug() << "looking for" << key;
+    QMap<QByteArray, ContentStructure>::const_iterator it = d->contents.constFind(key);
+    if (it == d->contents.constEnd()) {
+        return QStringList();
+    }
+
+    //kDebug() << "found" << key << "and the value is" << it.value().paths;
+    return it.value().paths;
 }
 
 QString PackageStructure::name(const char *key) const
@@ -392,8 +449,8 @@ void PackageStructure::setPath(const QString &path)
 
     if (valid) {
         QFileInfo info(basePath);
-        if (info.isDir()) {
-            basePath.append(QDir::separator());
+        if (info.isDir() && !basePath.endsWith('/')) {
+            basePath.append('/');
         }
         //kDebug() << "basePath is" << basePath;
     } else {
@@ -425,15 +482,15 @@ void PackageStructure::read(const KConfigBase *config)
 {
     d->contents.clear();
     d->mimetypes.clear();
-    KConfigGroup general(config, "");
+    KConfigGroup general(config, QString());
     d->type = general.readEntry("Type", QString());
-    d->contentsPrefix = general.readEntry("ContentsPrefix", d->contentsPrefix);
+    d->contentsPrefixPaths = general.readEntry("ContentsPrefixPaths", d->contentsPrefixPaths);
     d->packageRoot = general.readEntry("DefaultPackageRoot", d->packageRoot);
     d->externalPaths = general.readEntry("AllowExternalPaths", d->externalPaths);
 
     QStringList groups = config->groupList();
     foreach (const QString &group, groups) {
-        KConfigGroup entry(config, "");
+        KConfigGroup entry(config, group);
         QByteArray key = group.toAscii();
 
         QString path = entry.readEntry("Path", QString());
@@ -457,14 +514,14 @@ void PackageStructure::write(KConfigBase *config) const
 {
     KConfigGroup general = KConfigGroup(config, "");
     general.writeEntry("Type", type());
-    general.writeEntry("ContentsPrefix", d->contentsPrefix);
+    general.writeEntry("ContentsPrefixPaths", d->contentsPrefixPaths);
     general.writeEntry("DefaultPackageRoot", d->packageRoot);
     general.writeEntry("AllowExternalPaths", d->externalPaths);
 
     QMap<QByteArray, ContentStructure>::const_iterator it = d->contents.constBegin();
     while (it != d->contents.constEnd()) {
         KConfigGroup group = config->group(it.key());
-        group.writeEntry("Path", it.value().path);
+        group.writeEntry("Path", it.value().paths);
         group.writeEntry("Name", it.value().name);
         if (!it.value().mimetypes.isEmpty()) {
             group.writeEntry("Mimetypes", it.value().mimetypes);
@@ -482,12 +539,23 @@ void PackageStructure::write(KConfigBase *config) const
 
 QString PackageStructure::contentsPrefix() const
 {
-    return d->contentsPrefix;
+    return d->contentsPrefixPaths.isEmpty() ? QString() : d->contentsPrefixPaths.first();
 }
 
 void PackageStructure::setContentsPrefix(const QString &prefix)
 {
-    d->contentsPrefix = prefix;
+    d->contentsPrefixPaths.clear();
+    d->contentsPrefixPaths << prefix;
+}
+
+QStringList PackageStructure::contentsPrefixPaths() const
+{
+    return d->contentsPrefixPaths;
+}
+
+void PackageStructure::setContentsPrefixPaths(const QStringList &prefixPaths)
+{
+    d->contentsPrefixPaths = prefixPaths;
 }
 
 bool PackageStructure::installPackage(const QString &package, const QString &packageRoot)
@@ -528,10 +596,8 @@ void PackageStructure::setServicePrefix(const QString &servicePrefix)
 
 void PackageStructurePrivate::createPackageMetadata(const QString &path)
 {
-    if (metadata) {
-        delete metadata;
-        metadata = 0;
-    }
+    delete metadata;
+    metadata = 0;
 
     QString metadataPath(path + "/metadata.desktop");
     if (!QFile::exists(metadataPath)) {
@@ -551,15 +617,28 @@ PackageMetadata PackageStructure::metadata()
         if (fileInfo.isDir()) {
             d->createPackageMetadata(d->path);
         } else if (fileInfo.exists()) {
-            KZip archive(d->path);
-            if (archive.open(QIODevice::ReadOnly)) {
-                const KArchiveDirectory *source = archive.directory();
+            KArchive *archive = 0;
+            KMimeType::Ptr mimetype = KMimeType::findByPath(d->path);
+
+            if (mimetype->is("application/zip")) {
+                archive = new KZip(d->path);
+            } else if (mimetype->is("application/x-compressed-tar") ||
+                       mimetype->is("application/x-tar")|| mimetype->is("application/x-bzip-compressed-tar")) {
+                archive = new KTar(d->path);
+            } else {
+                kWarning() << "Could not open package file, unsupported archive format:" << d->path << mimetype->name();
+            }
+
+            if (archive->open(QIODevice::ReadOnly)) {
+                const KArchiveDirectory *source = archive->directory();
                 KTempDir tempdir;
                 source->copyTo(tempdir.name());
                 d->createPackageMetadata(tempdir.name());
             } else {
                 kWarning() << "Could not open package file:" << d->path;
             }
+
+            delete archive;
         }
     }
 
