@@ -1,20 +1,21 @@
-/*  This file is part of the KDE project
-    Copyright (C) 2006-2007 Kevin Ottens <ervin@kde.org>
+/*
+    Copyright 2006-2007 Kevin Ottens <ervin@kde.org>
 
     This library is free software; you can redistribute it and/or
-    modify it under the terms of the GNU Library General Public
-    License version 2 as published by the Free Software Foundation.
+    modify it under the terms of the GNU Lesser General Public
+    License as published by the Free Software Foundation; either
+    version 2.1 of the License, or (at your option) version 3, or any
+    later version accepted by the membership of KDE e.V. (or its
+    successor approved by the membership of KDE e.V.), which shall
+    act as a proxy defined in Section 6 of version 3 of the license.
 
     This library is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-    Library General Public License for more details.
-
-    You should have received a copy of the GNU Library General Public License
-    along with this library; see the file COPYING.LIB.  If not, write to
-    the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-    Boston, MA 02110-1301, USA.
+    Lesser General Public License for more details.
 
+    You should have received a copy of the GNU Lesser General Public
+    License along with this library. If not, see <http://www.gnu.org/licenses/>.
 */
 
 #include "powermanagement.h"
@@ -30,6 +31,9 @@ Solid::PowerManagementPrivate::PowerManagementPrivate()
     : managerIface("org.freedesktop.PowerManagement",
                    "/org/freedesktop/PowerManagement",
                    QDBusConnection::sessionBus()),
+      policyAgentIface("org.kde.Solid.PowerManagement.PolicyAgent",
+                       "/org/kde/Solid/PowerManagement/PolicyAgent",
+                       QDBusConnection::sessionBus()),
       inhibitIface("org.freedesktop.PowerManagement.Inhibit",
                    "/org/freedesktop/PowerManagement/Inhibit",
                    QDBusConnection::sessionBus())
@@ -53,6 +57,10 @@ Solid::PowerManagementPrivate::~PowerManagementPrivate()
 {
 }
 
+Solid::PowerManagement::Notifier::Notifier()
+{
+}
+
 bool Solid::PowerManagement::appShouldConserveResources()
 {
     return globalPowerManager->powerSaveStatus;
@@ -87,8 +95,15 @@ void Solid::PowerManagement::requestSleep(SleepState state, QObject *receiver, c
 
 int Solid::PowerManagement::beginSuppressingSleep(const QString &reason)
 {
-    QDBusReply<uint> reply = globalPowerManager->inhibitIface.Inhibit(
-        QCoreApplication::applicationName(), reason);
+    QDBusReply<uint> reply;
+    if (globalPowerManager->policyAgentIface.isValid()) {
+        reply = globalPowerManager->policyAgentIface.AddInhibition(
+            (uint)PowerManagementPrivate::InterruptSession,
+            QCoreApplication::applicationName(), reason);
+    } else {
+        // Fallback to the fd.o Inhibit interface
+        reply = globalPowerManager->inhibitIface.Inhibit(QCoreApplication::applicationName(), reason);
+    }
 
     if (reply.isValid())
         return reply;
@@ -98,7 +113,39 @@ int Solid::PowerManagement::beginSuppressingSleep(const QString &reason)
 
 bool Solid::PowerManagement::stopSuppressingSleep(int cookie)
 {
-    return globalPowerManager->inhibitIface.UnInhibit(cookie).isValid();
+    if (globalPowerManager->policyAgentIface.isValid()) {
+        return globalPowerManager->policyAgentIface.ReleaseInhibition(cookie).isValid();
+    } else {
+        // Fallback to the fd.o Inhibit interface
+        return globalPowerManager->inhibitIface.UnInhibit(cookie).isValid();
+    }
+}
+
+int Solid::PowerManagement::beginSuppressingScreenPowerManagement(const QString& reason)
+{
+    if (globalPowerManager->policyAgentIface.isValid()) {
+        QDBusReply<uint> reply = globalPowerManager->policyAgentIface.AddInhibition(
+            (uint)PowerManagementPrivate::ChangeScreenSettings,
+            QCoreApplication::applicationName(), reason);
+
+        if (reply.isValid())
+            return reply;
+        else
+            return -1;
+    } else {
+        // No way to fallback on something, hence return failure
+        return -1;
+    }
+}
+
+bool Solid::PowerManagement::stopSuppressingScreenPowerManagement(int cookie)
+{
+    if (globalPowerManager->policyAgentIface.isValid()) {
+        return globalPowerManager->policyAgentIface.ReleaseInhibition(cookie).isValid();
+    } else {
+        // No way to fallback on something, hence return failure
+        return false;
+    }
 }
 
 Solid::PowerManagement::Notifier *Solid::PowerManagement::notifier()
